function W = buildTensor( U, HOM1, HOM2, HOM3)
% build the tensor which is used to compute the probability of, given a
% specific input configuration, an output configuration for the netwerk U
% This function follows the definitions as used in Tichy PRL 2015
% 
% Input
%   U       complex valued matrix, the submatrix you probe with the photons
%   HOM     array of 2 photon HOM visibilities 
% 
% Output
%   W       nxnxn complex tensor, to be used as input in the MEX permT
%           function (Tichy's formula compiled C for Matlab)
% 
% Reinier vd Meer (2021)

HOM = [HOM1, HOM2, HOM3];

% setup the distinguishability matrix
S = eye(length(HOM));

% to do: check permutation order & generalize to n-photons
S = S + sqrt(HOM(1)) * [0 0 0; 0 0 1; 0 1 0];
S = S + sqrt(HOM(2)) * [0 0 1; 0 0 0; 1 0 0];
S = S + sqrt(HOM(3)) * [0 1 0; 1 0 0; 0 0 0];

% get the permutation arrays
Config1 = 1 : length(U);
Config2 = 1 : length(U);

% actually build the nxnxn tensor (n=2)
for kkk = 1 : length(U)
    kValue = Config1(kkk);
    
    for lll = 1 : length(U)
        lValue = Config1(lll);
        
        for jjj = 1 : length(U)
            jValue = Config2(jjj);
            W(kkk,lll,jjj) = U(kValue,jValue) .*conj(U(lValue,jValue))...
                    *S(lValue,kValue);
        end
    end
end
end