function [] = FancyPlot(FigureNumber,Paper, xSize, ySize, fontSize, linWidth, mrkSize)
%	FancyPlot makes the plot fancy :)
%   INPUT:
%   Figurenumber [int]  - The figurenumber of the plot
%   Paper [boolean]     - 0 -> plot will be optimised for a presentation
%                       - 1 -> PRL / PRA 1 column figure
%                       - 3 -> PRL / PRA 2 column figure
%                       - 4 -> PRL / PRA 1 colum inset (1/4)
%                       - 5 -> PRL / PRA 1 column figure (2/3 height) 
%                       - 6 -> PRL / PRA 1 column 80% height
%   Optional: (if used, then FigureNumber is ignored. All of these must be
%              specified, or none of them)
%    xSize              in cm (0.1 is used as margin)
%    ySize              in cm (0.1 is used as margin)
%    fontSize           fontsize of text
%    linWidth           width of the lines
%    mrkSize            size of the markers
% 
%   OUTPUT:
%   A fancy plot

% in case no figure number is provided
if nargin == 1
    Paper = FigureNumber;
    FigureNumber = get( gcf, 'Number');
end

% Test if input is okay
if mod(FigureNumber,1) ~= 0
    error('Figurenumbers should be integers!')
end


% General Settings
Fig = figure(FigureNumber);
set( gca, 'color', [1 1 1]);
set( gcf, 'color', [1 1 1]);
hold on;
% grid on;
box on;

if nargin > 2
    X = xSize;
    Y = ySize;
    xMargin = 0.1;
    yMargin = 0.1;
    set( gca, 'FontSize', fontSize);
    set( 0, 'DefaultAxesLineWidth',linWidth,...
        'DefaultLineMarkerSize',mrkSize)
else

if Paper == 0
    Y = 21;                                     % Define plotsize
    X = 29.7;
    xMargin = 0;                                % Set the margins
    yMargin = 0;
    set( gca, 'fontsize', 18);                  % Define fontsize
    set( 0, 'DefaultAxesLineWidth',2,'DefaultLineMarkerSize',5)
elseif Paper == 1
    % PRL / PRA 1 column figure
    Y = 8.6;
    X = 8.6;
    xMargin = 0.01;
    yMargin = 0.01;
    set( gca, 'fontsize', 10, 'linewidth', 1);
elseif Paper == 2
    % PRL / PRA 2 column figure
    % square plot
    Y = 15.5;
    X = 17.8;
    xMargin = 0.01;
    yMargin = 0.01;
    set( gca, 'FontSize', 10, 'linewidth', 1);
elseif Paper == 3
    % PRL / PRA 2 column figure
    % horizontal box plot
    Y = 8.6;
    X = 17.8;
    xMargin = 0.01;
    yMargin = 0.01;
    set( gca, 'FontSize', 10, 'linewidth', 1);
elseif Paper == 4
    Y = 8.6/2.5;
    X = 8.6/2.5;
    xMargin = 0.0;
    yMargin = 0.0;
    set( gca, 'FontSize', 8, 'linewidth', 1);
elseif Paper == 5
    % PRL / PRA 1 column figure 66%
    Y = 8.6*2/3;
    X = 8.6;
    xMargin = 0.01;
    yMargin = 0.01;
    set( gca, 'fontsize', 10, 'linewidth', 1);
elseif Paper == 6
    % PRL / PRA 1 column figure 80% height
    Y = 8.6*.8;
    X = 8.6;
    xMargin = 0.01;
    yMargin = 0.01;
    set( gca, 'fontsize', 10, 'linewidth', 1);
elseif Paper == 7
    % PRL / PRA, subplot 3 in a 2 column figure
    % 80% height
    Y = 8.6*.8;
    X = 5.9;
    xMargin = 0.01;
    yMargin = 0.01;
    set( gca, 'fontsize', 10, 'linewidth', 1);
else
    error('Incorrect input for "paper"')
end
end


xSize = X - 2 * xMargin;                        % figure size on paper (widht & height)
ySize = Y - 2 * yMargin;
set( FigureNumber, 'Units', 'centimeters', 'Position', [0 0 xSize ySize])
movegui( FigureNumber, 'center')


set(Fig, 'PaperUnits', 'centimeters');
set(Fig, 'PaperSize', [X Y]);
set(Fig, 'PaperPosition', [xMargin yMargin xSize ySize]);

set(gca, 'XMinorTick', 'off', 'YMinorTick', 'off')


end