clear all
% Making vacuum, 1,2 & 3 photon state vectors
vac = [1;0;0;0];
ph1 = [0;1;0;0];
ph2 = [0;0;1;0];
ph3 = [0;0;0;1];

in = kron(ph1,kron(ph1,kron(ph1,vac))); % |1110>

nt = 1000; 
tvec = linspace(0,2*pi,nt);

for j = 1:nt

out = BH4(tvec(j),4,1)*in;

[s,u,v] = SchmidtDecomposition(out,[16,16],0); % Even bipartition (subsystem A=modes 1 & 2, subsystem B = modes 3& 4

smax(j) = s(1);
S2(j) = -sum(abs(s).^2.*log2(abs(s).^2)); % Entanglement entropy

[s,u,v] = SchmidtDecomposition(out,[4,64],0); % A = mode 1, B = modes 1,2,&3 

smax1(j) = s(1);
S1(j) = -sum(abs(s).^2.*log2(abs(s).^2)); %Entanglement entropy
end
%%
figure(2)
semilogx(tvec,smax1.^2)
hold on
semilogx(tvec,smax.^2)

semilogx([.1 .2 .5 1 2 5], [.766 .734 .684 .582 .652 .675], 'x', 'MarkerSize', 10, 'LineWidth', 3)
hold off
xlabel('t');ylabel('\lambda_{max}^2');
legend('Even Bipartition ( 12 | 34 )','Uneven Bipartition ( 1 | 234 )')

figure(3)
plot(tvec,S2,tvec,S1)
xlabel('t');ylabel('E(\rho_{AB})');
legend('A=12, B=34','A = 1, B=234');
