clear all
close all

addpath([pwd,'\Functions\']);

% Making vacuum, 1,2 & 3 photon state vectors
vac = [1;0;0;0];
ph1 = [0;1;0;0];
ph2 = [0;0;1;0];
ph3 = [0;0;0;1];

in = kron(ph1,kron(ph1,kron(ph1,vac))); % |1110>

nt = 2; 
tvec = linspace(0,1,nt);

for j = 1:nt

out = BH4(tvec(j),4,1)*in;

r=linspace(0,5,100);
theta=linspace(0,2*pi,100);
[r,theta]=meshgrid(r,theta);

% create a set of coordinates for the Fock state representation of the
% density matrix
Fockn=0:3;
[m,n]=meshgrid(Fockn);

r1 = TrX(out*out',[2,3,4],[4,4,4,4]);
r2 = TrX(out*out',[1,3,4],[4,4,4,4]);
r3 = TrX(out*out',[1,2,4],[4,4,4,4]);
r4 = TrX(out*out',[1,2,3],[4,4,4,4]);



%first we do it in polar

x=r.* cos(theta);
y=r.* sin(theta);
z1=WignerFromRhoPolar(r1,r,theta);
z2=WignerFromRhoPolar(r2,r,theta);
z3=WignerFromRhoPolar(r3,r,theta);
z4=WignerFromRhoPolar(r4,r,theta);

figure(1)
subplot(nt,4,j)
surf(x,y,real(z1))
zlim([-0.6 0.4])
xlim([-2 2])
ylim([-2 2])
xlabel('p');ylabel('q'),zlabel('W');
subplot(nt,4,j+1)
surf(x,y,real(z2))
zlim([-0.6 0.4])
xlim([-2 2])
ylim([-2 2])
xlabel('p');ylabel('q'),zlabel('W');
subplot(nt,4,j+2)
surf(x,y,real(z3))
zlim([-0.6 0.4])
xlim([-2 2])
ylim([-2 2])
xlabel('p');ylabel('q'),zlabel('W');
subplot(nt,4,j+3)
surf(x,y,real(z4))
zlim([0 0.7])
xlim([-2 2])
ylim([-2 2])
xlabel('p');ylabel('q'),zlabel('W');


%then in Cartesian

x=linspace(-3,3,101);
y=linspace(-4,4,101);
[x,y]=meshgrid(x,y);

[theta,r]=cart2pol(x,y);
z1=WignerFromRhoPolar(r1,r,theta);
z2=WignerFromRhoPolar(r2,r,theta);
z3=WignerFromRhoPolar(r3,r,theta);
z4=WignerFromRhoPolar(r4,r,theta);



figure(2)
subplot(nt,4,(j-1)*4+1)
surf(x,y,real(z1),'EdgeColor','None')
zlim([-0.6 0.4])
xlim([-2 2])
ylim([-2 2])
xlabel('p');ylabel('q'),zlabel('W');
subplot(nt,4,(j-1)*4+2)
surf(x,y,real(z2),'EdgeColor','None')
zlim([-0.6 0.4])
xlim([-2 2])
ylim([-2 2])
xlabel('p');ylabel('q'),zlabel('W');
subplot(nt,4,(j-1)*4+3)
surf(x,y,real(z3),'EdgeColor','None')
zlim([-0.6 0.4])
xlim([-2 2])
ylim([-2 2])
xlabel('p');ylabel('q'),zlabel('W');
subplot(nt,4,(j-1)*4+4)
surf(x,y,real(z4),'EdgeColor','None')
zlim([0 0.7])
xlim([-2 2])
ylim([-2 2])
xlabel('p');ylabel('q'),zlabel('W');

xt=x';
xv=xt(1:length(xt));
yv=y(1:length(y));
Pp1=trapz(yv,real(z1));
Pq1=trapz(xv,real(z1),2);
Pp2=trapz(yv,real(z2));
Pq2=trapz(xv,real(z2),2);
Pp3=trapz(yv,real(z3));
Pq3=trapz(xv,real(z3),2);
Pp4=trapz(yv,real(z4));
Pq4=trapz(xv,real(z4),2);

figure(3)
subplot(2,1,1);
plot(xv,Pp1);
xlabel('p');ylabel('Pp');
subplot(2,1,2);
plot(yv,Pq1);
xlabel('q');ylabel('Pq');
end
