 % process the fiber splitter data
clc
close all
clear 


% settings
nTrigger = 1;
trigCh = 13;
doNormalize = 1;
doPlot = 0;
nPhotons = 3;
nModes = 4;


% file selection
Folder0 = 'M:\Data\Thermalization\20210531_Certification\';
nameData = '20210*_data_Thermalization_Certification_';
% type = 'BoseHubbard';
type = 'Fourier';
% type = 'Haar';
% type = 'Identity';
HaarNr = 1; 

% identify all state vectors
states = getStates( nPhotons, nModes);
strState = getStateString( states);
nStates= size( states,1);


% where to save the data
FolderS = 'M:\Data\Thermalization\20210531_Certification\Results\';


% get the weightfactors to correct against biased sampling
fiberFolder = 'M:\Data\Thermalization\20210531_Certification\FiberSplitter\';
fiberName = '20210*_Weightfactor_Fibersplitter_batch';


% HOM visibilities
HOM = [.89 .75 .75];  % check if still correct!!
% HOM1  input mode 2 - 3
% HOM2  input mode 1 - 3
% HOM3  input mode 1 - 2


% the names of the different types of measurements
name{1} = 'F';
name{2} = 'U-1';
name{3} = 'U-1F';
name{4} = 'U-1_HOM12';
name{5} = 'U-1_HOM13';
name{6} = 'U-1_HOM23';
sName{1} = 'F';
sName{2} = 'U1';
sName{3} = 'U1F';
sName{4} = 'U1_HOM_12';
sName{5} = 'U1_HOM_13';
sName{6} = 'U1_HOM_23';


% setup the parameters for BoseHubbard or Haar
if strcmp(upper(type), 'BOSEHUBBARD')
    Folder = [Folder0, 'Bose-Hubbard\'];
    timeRange = 1:6;
elseif strcmp(upper(type), 'HAAR')
    Folder = [Folder0, 'Haar\'];
    timeRange = HaarNr : 25 : 150;
elseif strcmp(upper(type), 'FOURIER')
    Folder = [Folder0, 'Fourier\'];
    timeRange = 1;
    
    clear name sName
    name{1} = 'Fourier';
    sName{1}= 'Fourier';
elseif strcmp(upper(type),'IDENTITY')
    Folder = [Folder0, 'Identity\'];
    timeRange = 1;
    clear name sName
    sName{1} = 'Identity';
    name{1}  = 'Identity';
end
    



% define experimental parameters
nBatches = 3;
nTimeSteps = length(timeRange);
nOutputs = 220;
nStates  = size( states,1);


% loop through the data and process it
for nn = 1 : length(name)
    
   
    % loop through all batches
    % each batch is a short measurement which is repeated nBatches times
    % hence I should collect al data and append it before processing
    cCounts{ nBatches} = zeros( nTimeSteps, nOutputs);
    Pcor.(sName{nn}){ nTimeSteps} = zeros( nBatches, nStates);
    Praw.(sName{nn}){ nTimeSteps} = zeros( nBatches, nStates);

    for bb = 1 : nBatches
        
        
        % load in the weight factors
        weightfactor(bb,:) = getWeightfactor(fiberFolder,...
            [fiberName, num2str(bb),'.mat']);
        
        
        clear data
        % loop through all timesteps in this batch
        ind = 1;
        for ii = timeRange
            
            
            % load in the data
            if strcmp(upper(type), 'FOURIER')
                filename = [nameData, name{nn},...
                    '_batch_', num2str(bb), '.mat'];
            elseif strcmp(upper(type), 'IDENTITY')
                filename = [nameData, name{nn},...
                    '_batch_', num2str(bb), '.mat'];
            else
                filename = [nameData, name{nn}, '_timestep', num2str(ii),...
                    '_batch_', num2str(bb),'.mat'];
            end
            files = dir([Folder, filename]);
            if length(files) > 1
                warning('Multiple datasets found, only take the 1st one')
            elseif length(files) == 0
                warning(['No files found, skip this instance (', name{nn}, ', batch ',num2str(bb), ', timestep ', num2str(ii), ')'])
                ind = ind+1;
                continue  
            end
            data{ind} = load( [Folder, files(1).name]);
            
            
            % get the singles
            [sCounts, totSingles, singleNames] = getSingles(...
                data{ind}.data.Singles, nTrigger);
            nSingleName = length( singleNames);
            
            
            % get the coincidences
            [cCounts{ind}(bb,:), totCoin, coinNames] = getCoincidences(...
                data{ind}.data.Coincidences, data{ind}.data.IntegrationTime);
            nCoinName = length( coinNames);
            

            % unnormalized probabilities to state
            [Praw.(sName{nn}){ind}(bb,:), Pcor.(sName{nn}){ind}(bb,:)]...
                = getProbabilities( data{ind}.data.Coincidences,...
                    weightfactor(bb,:), states, nStates, nPhotons,...
                    trigCh, nModes, data{ind}.data.IntegrationTime);
            
            
            
            % start plotting the data
            if doPlot == 1
                plotSingles( sCounts, totSingles, doNormalize, singleNames)
                plotCoincidences( cCounts, totCoin, nCoinName,...
                    nSingleName, Praw, Pcor, nStates, strState)
            end
            ind = ind+1;
            
        end
    end
    
    % now combine all counts / Pcorr from all batches and compute Pk
    % loop through all time steps
    for ii = 1 : nTimeSteps
        
        % normalize Pcor
        temp = sum( Pcor.(sName{nn}){ii});
        PcorN.(sName{nn}){ii} = temp / sum(temp);
        
        % get Pk distribution for each mode
        Pk.(sName{nn}){ii} = zeros(nModes,nModes);
        
        % loop through all modes
        for jj = 1 :nModes
            % only do it if there is actually some data
            if max( max( PcorN.(sName{nn}){ii} )) >0
                Pk.(sName{nn}){ii}(jj,:) =  getPk_Experiment([PcorN.(sName{nn}){ii}',states], nPhotons, jj);
            end
        end
    end
end


%% Certification
close all
clc
bRange = 1:4;
epsilon = 0.9;
doSave = 0;

[p d N, F] = plotCertificationFidelity( Praw, bRange,...
                epsilon, nTimeSteps, sName, nBatches, type, doSave, FolderS)



%% make the plot of Pk
% 1 plot per output mode and save it
clc
close all

if strcmp( upper(type), 'FOURIER')
    for ii = 1 : 4
        plotPk( Pk.Fourier, nPhotons, 1, 1, 'Fourier')
        legend('Fourier (Theory)',name{1})
        nameStr = [type, ', F, mode ', num2str(ii)];
        text( -0.5, 2.1, nameStr,...
            'units', 'normalized', 'HorizontalAlignment', 'center')
        saveStr = [type, '_Pk_F_mode_', num2str(ii)];
%         title('Partial distinguishable theory')
%         saveas(gcf,[FolderS, saveStr, '_partialDistinguishable.pdf'])
%         saveas(gcf,[FolderS, saveStr, '.pdf'])
    end
else
    for ii = 1 %: 4
        plotPk( Pk.F, nPhotons, 6, 1, 'Fourier')
        legend('Fourier',name{1}, 'Position', [.818,.37,.1667,.113])
        nameStr = [type, ', F, mode ', num2str(ii)];
        text( -0.5, 2.1, nameStr,...
            'units', 'normalized', 'HorizontalAlignment', 'center')
        saveStr = [type, '_Pk_F_mode_', num2str(ii)];
%         saveStr = [type, '_Pk_F_mode_', num2str(ii), '_distinguishable'];
%         saveas(gcf,[FolderS, saveStr, '.pdf'])
        
        plotPk( Pk.U1, nPhotons, 6, 1, 'Identity')
        legend('Identity',name{2}, 'Position', [.818,.37,.1667,.113])
        nameStr = [type, ', U^{-1}, mode ', num2str(ii)];
        text( -0.5, 2.1, nameStr,...
            'units', 'normalized', 'HorizontalAlignment', 'center')
        saveStr = [type, '_Pk_U1_mode_', num2str(ii)];
%         saveStr = [type, '_Pk_U1_mode_', num2str(ii), '_distinguishable'];
%         saveas(gcf,[FolderS, saveStr, '.pdf'])
        
        plotPk( Pk.U1F, nPhotons, 6, 1, 'Fourier')
        legend('Fourier',name{3}, 'Position', [.818,.37,.1667,.113])
        nameStr = [type, ', U^{-1}F, mode', num2str(ii)];
        text( -0.5, 2.1, nameStr,...
            'units', 'normalized', 'HorizontalAlignment', 'center')
        saveStr = [type, '_Pk_U1F_mode_', num2str(ii)];
%         saveStr = [type, '_Pk_U1F_mode_', num2str(ii), '_distinguishable'];
%         saveas(gcf,[FolderS, saveStr, '.pdf'])
        pause(0.1)
        
    end
end


%% Make a plot of the microstate distribution
% HOM = ones(1,3);
HOM = [.89 .75 .75]
close all
clc
batchRange = 8;
doSave = 0;
plotMicroStateCertification( Pcor, HOM, batchRange, nBatches,...
    nStates, nTimeSteps, type, sName, strState, doSave, FolderS)


%% Plot the weightfactors over time
plotWeightfactor( weightfactor)


%% plot reference HOM dips
plotReferenceHOM( [Folder0, 'Reference HOM dips\'])



%% function space
function states = getStates( n, m)
% computes all the state vectors for a n photons in a m mode interferomter
% output:   
%   states      a list of all state vectors

% all possible number of states is a multiset
nStates = nchoosek( n+m-1,n);

% allocate memory
states = zeros( nStates, m);
modes_list = nmultichoosek(1:m, n);   

% loop through all possibilies
for ii = 1 : nStates
    list = modes_list(ii,:);
    for jj = 1 : n
        states(ii,list(jj)) = states(ii,list(jj)) + 1;
    end
end
end

function strState = getStateString( states)
% converts the matrix of states into a structure of strings
% this is used as xticklabel when you plot the distribution as function of
% all the output state vectors

[nStates, nModes] = size(states);
for ii = 1 : nStates
    strState{ii} = [];
    for jj = 1 : nModes
        strState{ii} = [strState{ii}, num2str(states(ii,jj))];
    end
end
end

function list = stateToChannel( state)
% converts a state into the possible combinations of channels of the
% Swabian. The state should contain max 3 photons over 4 modes
% Output:
%   List    matrix of all channel combinations which belong to this state
%           each row is a combination of channels

% hardcoded the channels which belong to a certain output mode
splitter1 = [9 10 11];
splitter2 = [6 7 12];
splitter3 = [5 8 14];
splitter4 = [1 2 4];

% sanity check:
if sum( state) ~= 3
    state
    warning('total number of photons ~= 3')
end

list = [];
% loop through the first mode
a = nchoosek( splitter1, state(1));
for ii = 1 : size(a,1)
    % 2nd mode
    b = nchoosek(splitter2, state(2));
    for jj = 1: size(b,1)
        % 3rd mode
        c = nchoosek(splitter3, state(3));
        for kk = 1 : size(c,1)
            % 4th mode
            d = nchoosek( splitter4, state(4));
            for ll = 1 : size(d,1)
                list = [list; a(ii,:),b(jj,:),c(kk,:),d(ll,:)];
            end
        end
    end
end
% sort the list to ascending channel number
list = sort( list,2);
end

function [sCounts, totSingles, singleNames] = getSingles( Singles, nTrigger)
% extract the single counts
% input:
%   Singles         a struct with all the singles
% output:
%   sCounts         array with average counts in each channel
%   totSingles      the total number of counts
%   singleNames     structure with strings of each channelnames

% extract singles
singleNames = fieldnames( Singles);
nSingleName = length( singleNames);

sCounts = zeros( 1, nSingleName);
for jj = 1 : nSingleName - nTrigger
    sCounts(jj) = sum( Singles.(singleNames{jj}));
end
totSingles = sum( sCounts);
end

function [cCounts, totCoin, coinNames] = getCoincidences(Coincidences, intTime)
% extract the single counts
% input:
%   Coincidences    a struct with all the coincidences
%   intTime         the integration time (in seconds)
% output:
%   cCounts         array with total counts in each channel
%   totCoin         the total number of coincidences counts
%   coinNames       structure with strings of each channelnames

if nargin == 1
    intTime = 1;
end

% extract coincidences
coinNames = fieldnames( Coincidences);
nCoinName = length(coinNames);

cCounts = zeros( 1, nCoinName);
for jj = 1 : nCoinName
    cCounts(jj) = sum( Coincidences.(coinNames{jj}))*intTime;
end
totCoin = sum( cCounts);
end

function weightfactor = getWeightfactor(Folder, FileName)
% extract the weightfactors for the fiber splitter
files = dir([Folder, FileName]);

if length(files) > 1
    warning('multiple fiber characterization files found')
end
A = load([Folder,files(1).name]);
weightfactor = A.W;
end

function [Praw, Pcor] = getProbabilities( Coincidences, weightfactor, states, nStates, nPhotons, trigCh, nModes, intTime)
% get the biased (Praw) and unbiased (Pcorr) probabilities associated with
% each output state. 
% Input:
%   Coincidences    struct with all coincidence data
%   weightfactor    array with the relative weights of each detector
%                   (used to correct for biased sampling)
%   states          list of all the output states
%   nStates         total number of output states
%   nPhotons        total number of photons in experiment (excl. herald)
%   trigCh          the swabian channel of the trigger/herald photon
%   nModes          # modes of the chip
%   intTime         Integration time of the measurement, in seconds
% 
% output:
%   Praw            Probability uncorrected for the biased sampling
%   Pcor            Probability   corrected for the biased sampling

% allocate memory
Praw = zeros(1,nStates);
Pcor = zeros(1,nStates);

%loop through all the states
for nn = 1 : nStates
    
    % get a list of all channel combinations associated with the state
    list = stateToChannel( states(nn,:));
  
    for mm = 1 : size(list,1)
        % build up the string of the fieldname so we can extract the data
        str = '';
        for kk = 1 : nPhotons
            str = [str, 'C', num2str(list(mm,kk))];
        end
        str = [str, 'C', num2str(trigCh)];
        
        % now add the probabilities up (still biased!)
        Praw(nn) = Praw(nn) + sum(Coincidences.(str))*intTime;
    end
   
end

% Apply correction against biased sampling
correction = getCorrection(states, nModes, weightfactor);
correction = correction';  %column vector to row vector
Pcor = Praw./correction;

% % Convert from rates to probability
% Praw = Praw / sum(Praw);
Pcor = Pcor / sum(Pcor);
end

function correction = getCorrection(states, nModes, weightfactor)

% hardcoded the channels which belong to a certain output mode
splitter1 = [9 10 11];
splitter2 = [6 7 12];
splitter3 = [5 8 14];
splitter4 = [1 2 4];

splitterlist = [splitter1; splitter2; splitter3; splitter4];

correction = ones(size(states,1),1);
for ii = 1:size(states,1)
	for jj = 1:nModes
        
        if states(ii,jj) == 0
            correction(ii) = correction(ii)*1;
        else
            list = nchoosek(splitterlist(jj,:),states(ii,jj));
            Pk_nMode = 0;
            for kk = 1:size(list,1)
                Pk_nMode = Pk_nMode + factorial(states(ii,jj))*prod(weightfactor(list(kk,:)));
            end
            correction(ii) = correction(ii)*Pk_nMode;
        end
        
    end

end

% At this point, we have corrected for multinomial distribution introduced
% by fiber beam splitters (see Feito et al.)

end

function P_Canonical = CanonicalState(nModes,nPhotons)
% get the Pk distribution for the Canonical state 
    M = nModes;
    N = nPhotons;
    for k = 0:1:N
        P_Canonical(k+1) = nchoosek(M+N-k-2,N-k)/nchoosek(M+N-1,N);
    end
    
end

function [P_Fourier, P] = FourierState( nModes, nPhotons, mode0, threepoint, HOM)
% get the Pk distribution for a Fourier state for outputmode mode0

if nargin == 4
    HOM = ones(1,3);
end
if nargin < 4
    HOM = ones(1,3)
    threepoint = 0;
end

% get the input state & Fourier matrix
input = 1:nPhotons;
% if threepoint correlator == 1: set remaning channels to go straight
if threepoint == 1
    F = eye(nModes);
    F(1:nPhotons,1:nPhotons) = 1 / sqrt(nPhotons) * dftmtx( nPhotons);
else
    F = 1 / sqrt(nModes) * dftmtx( nModes);
end

states = getStates( nPhotons, nModes);
states0= states;
nStates= size( states,1);

temp = [zeros( nStates,1), states];
for ii = 1 : nStates
    
    ind = 1;
    for nn = 1 : nModes
        while states0(ii,nn) > 0
            output(ind) = nn;
            ind = ind+1;
            states0(ii,nn) = states0(ii,nn)-1;
        end
    end
    
    % get the probability
    if min(HOM) == 1
        % ideal case
        P = abs(perm( F(output,input))).^2;
    else
        % with partial distinguishabiliy
        W = buildTensor( F(output,input), HOM(1), HOM(2), HOM(3));
        P = abs(permT(W));
    end
    
    % correct for multiplicity
    Pnorm = P / prod( factorial( states(ii,:)));
    temp(ii,1) = Pnorm;
end

P = temp(:,1);

% get Pk for Fourier
P_Fourier = getPk_Experiment( temp, nPhotons, mode0);
end

function Pk_exp = getPk_Experiment(list_states, n, mode0)
% Gets Pk for the experimental data, i.e., the distribution of photons
% in one output mode of the chip
% input:   
%   data            a list of all states and the rate
%                   Format:
%                   Rate    state (1x4 array), e.g. for the |1110> state:
%                   0.3 1 1 1 0
%   n               number of photons
%   mode0           the mode of interest (e.g., 1 or 2)
%   Output:
%   Pk_exp          Pk belonging of the state of interest

% allocate memory
Rk_experimental = zeros(1,n+1);

% run through all possible number of photons
for k = 0:1:n
    
    % loop through all possibles states
    for ii = 1:length(list_states)
        
        % something??
        if list_states(ii,mode0+1) == k  
            Rk_experimental(k+1) = Rk_experimental(k+1) + list_states(ii,1);
        end
    end
end

% normalize data
R_tot = sum(Rk_experimental);
Pk_exp = Rk_experimental./R_tot;
end


% certification
function [F p1 p2 d1 d2] = Certification( N1, N2, Nt1, Nt2, epsilon)
% Certification as defined in eq. 3.31 of Franks master thesis (p 22)

% get the probabilities
p1 = N1 /Nt1;
p2 = 1 - N2 /Nt2;

% get the bounds
d1 = getErrorBound( p1, N1, epsilon);
d2 = getErrorBound( p2, N2, epsilon);

% get the fidelity
F = p1 -d1 -9/4*(p2+d2);

end

function d = getErrorBound( p, N, epsilon)
% computes the errorbound on p1 and p2 (eq 3.32, p22) of Frank)

d = sqrt( (2*p*(1-p)) / (N * log( 1/sqrt(epsilon))));
end

function [p, d, N, F] = plotCertificationFidelity( Praw,...
                bRange, epsilon, nTimeSteps, sName, nBatches, type, doSave, FolderS)
% plot the lower bounds on the fidelities and returns arrays with all
% values (p, d, and N)
% 
% Input parameters 
%   Praw        A struture with all raw data (uncorrected for detection
%               efficiencies etc.). Data in counts (not normalized!)
%   bRange      An array with the batches you want to check (if > nBatches, 
%               use all data (not just individual batches)
%   epsilon     Probability that the fidelity is correct
%   nTimeSteps  # timesteps (typically 6)
%   sName       A structure with all types (e.g., 'Identity', or 'U1F')
%   nBatches    # batches you measured
%   type        A string stating the type of system you are checking:
%               'BoseHubbard', 'Fourier', 'Haar' or 'Identity'
%   doSave      If 1, make save the plot. Else: don't save
%   FolderS     The path to the folder where you want to save the data
% 
% Output
%   p           A cell array of length 2, p{1} = p1, p{2} = p2
%               Each cell is a matrix where the values are given per batch
%               (sorted by bRange) and the timestep
%   d           A cell array of length 2, d{1} = error bound 1, same as p
%   N
%   F


% in this case we only have the direct 3point Fourier matrix
% this one only processes all batches combined
if length( sName) ==1
    yData = sum(Praw.(sName{1}){1});
    N1  = 0;
    N1t = 1;
    N2  = yData(1) + yData(6) + yData(11) + yData(17) + yData(20);
    N2t = sum(yData);
    [F, p1, p2, d1, d2] = Certification( N1, N2, N1t, N2t, epsilon);
    
    N{1} = 0;
    N{2} = N2t;
    p{1} = 0;
    p{2} = p2;
    d{1} = 0;
    d{2} = d2;

    figure
    subplot(2,1,1)
    plot( 1, p{2}(end,:), '^--', 'color', [.85 .33 .1], 'MarkerSize', 6)
    titleStr = [type, '\_Certification\_Parameters'];
    title( titleStr, 'fontweight', 'normal')
    ylabel('Probability p_2')
    ylim([0 1])
    xlim([.5 1.5])
    set( gca, 'xtick', 1)
    FancyPlot(3)
    
    subplot(2,1,2)
    hold on
    plot( 1, N{2}(end,:), '^--', 'color', [.85 .33 .1], 'MarkerSize', 6)
    xlabel('Timestep')
    ylabel('Counts N_2')
    ylim([0 2.5e4])
    xlim([.5 1.5])
    set( gca, 'xtick', 1)
    FancyPlot(3)
    
    if doSave == 1
        saveStr = [type, '_Certification_Parameters'];
        saveas( gcf, [FolderS, saveStr, '.pdf'])
    end
else

% allocate memory
F = zeros( max(bRange),nTimeSteps);
p{1} = zeros( max(bRange),nTimeSteps);
P{2} = zeros( max(bRange),nTimeSteps);
d{1} = zeros( max(bRange),nTimeSteps);
d{2} = zeros( max(bRange),nTimeSteps);
N{1} = zeros( max(bRange),nTimeSteps);
N{2} = zeros( max(bRange),nTimeSteps);

% go through all selected batches
for bb = bRange
    
    % go through all timesteps
    for ii = 1 : nTimeSteps
        
        % loop through both measurements U^-1 U^-1F
        for nn = 2:3
            
            % if >nBatches: sum up all data
            if bb > nBatches
                yData = sum(Praw.(sName{nn}){ii});
            else
                % yData = Praw.(sName{nn}){ii}(bb,:);
                
                % alternative, include all counts thusfar
                yData = (Praw.(sName{nn}){ii}(1:bb,:));
                if size( yData,1) > 1
                    yData = sum(yData);
                end
            end
            
            % get the counts
            if nn == 2
                N1 = yData(6);
                N1t= sum(yData);
            elseif nn == 3
                N2 = yData(1) + yData(6) + yData(11) + yData(17) + yData(20);
                N2t= sum(yData);
            end
        end
        
        % compute the fidelity for this timestep / batch number
        [F(bb,ii),p1,p2,d1,d2] = Certification( N1, N2, N1t, N2t, epsilon);
        
        N{1}(bb,ii) = N1t;
        N{2}(bb,ii) = N2t;
        p{1}(bb,ii) = p1;
        p{2}(bb,ii) = p2;
        d{1}(bb,ii) = d1;
        d{2}(bb,ii) = d2;
    end
    
    if bb > nBatches
        legEntry{bb} = ['All batches'];
    else
        legEntry{bb} = ['Batch ', num2str(bb)];
    end
    
end

% plot everything
m = ['+','o','*','x','s','d','^','v','>','<','p','h'];

figure
hold on
% loop through all batches
for ii = 1 : max(bRange) 
    
    % give total of all a thicker marker size
    if ii > nBatches
        linWdth = 1.5;
        mrkSize = 8;
    else
        linWdth = 1;
        mrkSize = 6;
    end
    % actually plot
    plot( 1 : nTimeSteps, F(ii,:)', [m(mod(ii-1,length(m))+1)],...
        'MarkerSize', mrkSize, 'linewidth', linWdth)
end

% labels
legend( legEntry(bRange), 'location', 'east')
xlabel('Timestep')
ylabel('Fidelity')

% fix x and y axis
set( gca, 'xtick', 1:6,...
    'xlim', [0 8])
ylim([min(ylim) 1])

% title
titleStr = [type, '\_Certification\_\epsilon=', num2str(epsilon)];
title(titleStr, 'fontweight', 'normal')
FancyPlot(3)

if doSave == 1
    saveStr = [type, '_Certification_epsilon=', num2str(epsilon)];
    saveas( gcf, [FolderS, saveStr, '.pdf'])
end


if ii > nBatches
    figure
    subplot(2,1,1)
    hold on
    plot( 1 : nTimeSteps, p{1}(end,:), 'o-', 'MarkerSize', 6)
    plot( 1 : nTimeSteps, p{2}(end,:), '^--', 'MarkerSize', 6)
    titleStr = [type, '\_Certification\_Parameters'];
    title( titleStr, 'fontweight', 'normal')
    ylabel('Probability')
    legend('p_1','p_2')
    ylim([0 1])
    xlim([.5 nTimeSteps+1.5])
    FancyPlot(3)
    
    subplot(2,1,2)
    hold on
    plot( 1 : nTimeSteps, N{1}(end,:), 'o-', 'MarkerSize', 6)
    plot( 1 : nTimeSteps, N{2}(end,:), '^--', 'MarkerSize', 6)
    xlabel('Timestep')
    ylabel('Counts')
    legend('N_1','N_2')
    ylim([0 max(ylim)])
    xlim([.5 nTimeSteps+1.5])
    FancyPlot(3)
    
    if doSave == 1
        saveStr = [type, '_Certification_Parameters'];
        saveas( gcf, [FolderS, saveStr, '.pdf'])
    end
end
end
end


% plotting functions
function plotWeightfactor( weightfactor)
% plot the weightfactors
figure
clf

m = ['+','o','*','x','s','d','^','v','>','<','p','h'];

[nW, nT] = size( weightfactor);
hold on

for ii = 1 : nW
    plot( 1:nT, weightfactor(ii,:),...
        [m(mod(ii-1,length(m))+1)], 'markersize', 10)
    legEntry{ii} = ['Batch ', num2str(ii)];
end
legend(legEntry, 'Location', 'East');
xlabel('Output channel')
ylabel('Relative weight')
FancyPlot(3)
xlim([0 nT+5])
title('Weightfactor', 'fontweight', 'normal')
end

function plotSingles( sCounts, totSingles, doNormalize, singleNames)
figure()
clf
% first the singles
subplot(3,1,1)
if doNormalize == 1
    bar( sCounts/totSingles)
    ylim([0 1])
    ylabel('Relative singles')
else
    bar( sCounts)
    ylabel('Singles (Hz)')
end
text( 0.05, 0.91, ['Total counts = ', num2str( round(totSingles/1000)), ' kHz'], 'units', 'normalized')
set( gca, 'xtick', 1:length(sCounts), 'xticklabels', singleNames)
box on         
end

function plotCoincidences( cCounts, totCoin, nCoinName, nSingleName, Praw, Pcor, nStates, strState)
subplot(3,1,2)
if doNormalize == 1
    bar( cCounts/totCoin)
    ylabel('Relative coincidences')
else
    bar( cCounts)
    ylabel('Coincidences (Hz)')
end

box on
ylim([0 ceil(max(ylim)*10)/10])
xlim([0 max( nCoinName, nSingleName)+1])
text( 0.05, 0.91, ['Total counts = ', num2str( round(totCoin,2)), ' Hz'], 'units', 'normalized')
xlabel('Output Channel')
set( gca, 'xticklabels', [])


% plot biased probability
subplot(3,1,3)
hold on
bar( -Praw/totCoin)
bar( Pcor/totCoin)
hold off
ylabel('Probability')
xlabel('Output State')
set( gca, 'xtick', 1:nStates, 'xticklabels', strState,...
    'ytick', [-1 0 1], 'yticklabels', {'1', '0', '1'})
legend('Uncorrected', 'Corrected')
ylim([-1 1])
box on
end

function plotPk( Pk_data, nPhotons, nFiles, Modes, type)
% Input
%   Pk_data     Cell array with 1 cell per timestep
%   nPhotons    3
%   nFiles      6 (or 1 for Fourier matrix)
%   nModes      4
%   type        'Canonical'
%               'Fourier'
%               'Identity'
timesteps = [.1 .2 .5 1 2 5];

for ii = Modes
    figure()
    clf
    
    % make a subplot without white space between them
    if nFiles == 1
        nRows = 1;
        nCols = 1;
    else
        nRows = 2;
        nCols = ceil( nFiles/2);
    end
    [ha, ~] = tight_subplot(nRows, nCols, 0, [.18 0.1], [.08 .04]);
    FancyPlot( 3)
    set( gca, 'FontSize', 10)
    
    switch upper(type)
        case 'CANONICAL'
            [Theory,~]= CanonicalState(4,nPhotons);
        case 'FOURIER'
            [Theory,~] = FourierState( 4, nPhotons, ii, 1);
        case 'IDENTITY'
            Theory = [0 1 0 0];
        case 'NONE'
            Theory = [0 0 0 0];
    end
    
    
    for jj = 1 : nFiles
        % select the frame of the subplot
        axes(ha(jj))
        
        
        % plotting
        hold on
        bar(  0 : nPhotons, Theory, 'FaceColor', [200 200 200]/255)
        plot( 0 : nPhotons, Pk_data{jj}(ii,:)', 'o', 'color', 'blue')
        hold off
        
        % setup the axes
        set( gca, 'xtick', 0:nPhotons, 'ytick', [0 .5 1])
        xlim([-.5 nPhotons+.5])
        ylim([0 1.2])
        box on
        
        % setup the time title
        if nFiles > 1
            text( 1.5, 1.13, ['\tau = ', num2str(timesteps(jj),'%3.1f')],...
                'HorizontalAlignment', 'center')
        end
%         [dist,~] = getDistance( Pk_data{jj}(ii,:), Theory);
%         text( -.35, 1.13, ['D = ', num2str( round( dist,3), '%4.3f')])
        
        % set up xticks at the bottom panels
        if jj > nFiles/2 || nFiles == 1
            set( gca, 'xticklabels', {'0', '1', '2', '3'})
            if jj == 5 || nFiles == 1
                xlabel('Photon number k')
            end
        end
        % set up yticks at the left panels
        if mod(jj,nFiles/2) == 1 || nFiles == 1
            set( gca, 'yticklabels', {'0', '0.5', '1.0'})
            ylabel('Probability P(k)')
        end
        FancyPlot(3)
%         % setup the legend
%         if jj == nFiles
%             legend('Canonical', 'Indistinguishable','Distinguishable')
%         end
        
    end
end
end

function plotReferenceHOM(Folder)
% Folder = 'M:\Data\Thermalization\20210521_Certification\Reference HOM dips\';
figure
clf
subplot(2,1,1)
load([Folder,'20210528_data_Signal1_Idler2.mat'])
hold on
errorbar( data.PositionList, mean(data.Coincidences.C10C11C9C13),...
    std( data.Coincidences.C10C11C9C13)/sqrt(size(data.Coincidences.C10C11C9C13,1)), 'o')

load([Folder,'20210528_data_Signal1_Signal2.mat'])
errorbar( data.PositionList, mean(data.Coincidences.C10C11C9C13),...
    std( data.Coincidences.C10C11C9C13)/sqrt(size(data.Coincidences.C10C11C9C13,1)), 'o')
hold off
% xlim([5.65 5.85])
ylim([0 1.1*max(ylim)])
ylabel('Counts (Hz)')
legend('S_1-I_2','S_1-S_2')
FancyPlot(3)

subplot(2,1,2)
load([Folder,'20210528_data_Signal2_Idler2.mat'])
errorbar( data.PositionList, mean(data.Coincidences.C10C11/1e3),...
    std( data.Coincidences.C10C11/1e3)/sqrt(size(data.Coincidences.C10C11,1)), 'o')
ylim([0 1.1*max(ylim)])
% xlim([5.45 5.65])
xlabel('Stage position (mm)')
ylabel('Counts (kHz)')
FancyPlot(3)
end

function plotMicroStateCertification( Pcor, HOM, batchRange, nBatches,...
    nStates, nTimeSteps, type, sName, strState, doSave, FolderS)

timesteps = [.1 .2 .5 1 2 5];

% to set the correct #rows/cols in the subplot
if nTimeSteps == 1
    nRows = 1;
    nCols = 1;
else
    nRows = 2;
    nCols = 3;
end


% loop through all certification measurements
for nn = 1 : length(sName)
    
    Theory = zeros(1,20);
    % get the relevant theoreitcal prediction
    if strcmp( upper(sName{nn}), 'U1')
        % the inverse: should give unity
        Theory = [0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0];
    elseif strcmp( upper(sName{nn}), 'F') || strcmp( upper(sName{nn}), 'U1F')
        % the 3point fourier
        [~,b] = FourierState( 4, 3, 1, 1, HOM);
        Theory = b';
    % now all the HOM dip theory
    elseif strcmp( upper(sName{nn}), 'U1_HOM_12')
        Theory([3,12]) = HOM(3)/2;
        Theory(6) = 1-HOM(3);
    elseif strcmp( upper(sName{nn}), 'U1_HOM_13')
        Theory([5,8]) = HOM(2)/2;
        Theory(6) = 1-HOM(2);
    elseif strcmp( upper(sName{nn}), 'U1_HOM_23')
        Theory([5,8]) = HOM(1)/2;
        Theory(6) = 1-HOM(1);
    end
    
    % loop through all batches
    for bb = batchRange
        
        figure
        clf
        [ha, pos] = tight_subplot(nRows,nCols, 0, [.18 0.1], [.08 .04]);
        FancyPlot(3)
        
        % loop through all timesteps
        for ii = 1 : nTimeSteps
            axes(ha(ii))
            
            % get the relevant Pcor data
            if bb <= nBatches
                yData = Pcor.(sName{nn}){ii}(bb,:);
            else
                yData = sum(Pcor.(sName{nn}){ii});
            end
            
            hold on
            bar(  1:nStates, Theory, 'FaceColor', [200 200 200]/255)
            plot( 1:nStates, yData / sum(yData), 'o',...
                'Color', 'blue', 'MarkerSize', 6)
            hold off
            
            FancyPlot(3)
            set( gca, 'xtick', 1:nStates,...
                'xlim', [0 nStates+1],...
                'ylim', [0 1.1],...
                'ytick', [0 .5 1])
            
            % time 
            if nTimeSteps > 1
                text( 10, 1.03, ['\tau = ', num2str(timesteps(ii),'%3.1f')],...
                    'HorizontalAlignment', 'center')
            end
            
            % yaxis
            if mod(ii, nTimeSteps/nRows) == 1 || nTimeSteps == 1
                set( gca, 'yticklabels', {'0', '0.5', '1'});
                ylabel('P')
            else
                set( gca, 'yticklabels', '');
            end
            % x axis
            if ii > nTimeSteps/2 || nTimeSteps == 1
                set(gca, 'xticklabels', strState,...
                    'XTickLabelRotation', 90)
                if ii == 5 || nTimeSteps == 1
                    xlabel('Microstate')
                end
            else
                set(gca, 'xticklabels', '')
            end
        end
        
        % title of overall plot
        if bb <= nBatches
            batchNr = num2str(bb);
        else
            batchNr = 'all';
        end
        nameStr = [type, ', ', sName{nn}, ', Batch ', batchNr];
        if nTimeSteps >1
            text( -0.5, 2.1, nameStr,...
                'units', 'normalized', 'HorizontalAlignment', 'center', 'Interpreter', 'none')
            legend('Theory', 'Experiment', 'Position', [.818,.37,.1667,.113])
        else
            text( 0.5, 1.05, [sName{nn}, ', Batch ', batchNr],...
                'units', 'normalized', 'HorizontalAlignment', 'center', 'Interpreter', 'none')
            legend('Theory', 'Experiment')
        end
        
        % save the plot, auto check if partial distinguishable was used
        if doSave == 1
            saveName = [type, '_Microstate_', sName{nn}, '_Batch_', batchNr];
            if min(HOM) < 1
                saveName = [ saveName, '_distinguishable'];
            end
            saveas(gcf,[FolderS, saveName, '.pdf'])
        end
    end
end
end


% low level functions
function combs = nmultichoosek(values, k)
    %// Return number of multisubsets or actual multisubsets.
    if numel(values)==1 
        n = values;
        combs = nchoosek(n+k-1,k);
    else
        n = numel(values);
        combs = bsxfun(@minus, nchoosek(1:n+k-1,k), 0:k-1);
        combs = reshape(values(combs),[],k);
    end
end

function [avdDist, maxDist] = getDistance( A, B)
% computes the average distance and max distance

distElement = abs(A-B);
avdDist = mean( distElement);
maxDist = max( distElement);
end

function W = buildTensor( U, HOM1, HOM2, HOM3)
% build the tensor which is used to compute the probability of, given a
% specific input configuration, an output configuration for the netwerk U
% This function follows the definitions as used in Tichy PRL 2015

% HOM1  input mode 2 - 3
% HOM2  input mode 1 - 3
% HOM3  input mode 1 - 2

% setup the distinguishability matrix
S = eye(3);
S = S + sqrt(HOM1) * [0 0 0; 0 0 1; 0 1 0];
S = S + sqrt(HOM2) * [0 0 1; 0 0 0; 1 0 0];
S = S + sqrt(HOM3) * [0 1 0; 1 0 0; 0 0 0];

% get the permutation arrays
Config1 = 1 : length(U);
Config2 = 1 : length(U);

% actually build the nxnxn tensor (n=2)
for kkk = 1 : length(U)
    kValue = Config1(kkk);
    
    for lll = 1 : length(U)
        lValue = Config1(lll);
        
        for jjj = 1 : length(U)
            jValue = Config2(jjj);
            W(kkk,lll,jjj) = U(kValue,jValue) .*conj(U(lValue,jValue))...
                    *S(lValue,kValue);
        end
    end
end
end

