% process the fiber splitter data
clc
close all
clear 

addpath([pwd,'\Functions']);
addpath([pwd,'\Raw data']);

% settings
nTrigger = 1;
trigCh = 13;
doNormalize = 1;
doPlot = 0;
nPhotons = 3;
nModes = 4;


% file selection
Folder = [pwd,'\Raw data\'];
nameData = '20210317';
x = [0];
type = 'Identity';


% identify all state vectors
states = getStates( nPhotons, nModes);
strState = getStateString( states);
nStates= size( states,1);             % Got it
 
% get the weightfactors to correct against biased sampling
fiberFolder = [pwd,'\Raw data\'];
fiberName = '20210317_Thermalization_weightfactor_FBS_Characterization.mat';
weightfactor = getWeightfactor(fiberFolder, fiberName); % As measured by fiber BS


% loop through both distinguishable and indistinguishable
for indistinguishable = x
    
    % build the file names
    if indistinguishable == 1
%         strDist = 'Indistinguishable';
        strDist = 'Indistinguishable';
    else
        strDist = 'Distinguishable';
    end
    %name = [nameData, '_Thermalization_data_', strDist, '_', type, '_light*.mat'];
    name = ['Thermalization_data_', strDist, '_', type, '_*.mat'];
    
    % identify all data files
    files = dir([Folder, name]);
    nFiles = length(files);
    
    % loop through all the files
    for ii = 1 : nFiles
        % load in the data
        data{ii} = load( [Folder, files(ii).name]);
        
        
        % get the singles
        [sCounts, totSingles, singleNames] = getSingles( data{ii}.data.Singles, nTrigger);
        nSingleName = length( singleNames);
        
        
        % get the coincidences
        [cCounts, totCoin, coinNames] = getCoincidences( data{ii}.data.Coincidences);
        nCoinName = length( coinNames);
        
        % --- Up to this point OK
        
        % probabilities to state
        [Praw, Pcor] = getProbabilities( data{ii}.data.Coincidences, weightfactor, states, nStates, nPhotons, trigCh, nModes);
        % Actually count rates instead of probabilities
        Pcor_store{ii}.(strDist) = Pcor;
        
        sum(Praw);
        
        display(['Countrate is ',num2str( sum(Praw) ),' Hz and Corrected countrate is ',num2str( sum(Pcor) ),' Hz'])
        % get Pk distribution for each mode
        Pk_bias{ii}.(strDist) = zeros(nModes,nModes);
        Pk{ii}.(strDist) = zeros(nModes,nModes);
        for jj = 1 :nModes
            Pk_bias{ii}.(strDist)(jj,:) =  getPk_Experiment([Praw',states], nPhotons, jj); %Actually not Pk but rates!
            Pk{ii}.(strDist)(jj,:) =  getPk_Experiment([Pcor',states], nPhotons, jj);
        end
        
        
        % start plotting the data
        if doPlot == 1
        figure()
        clf
        % first the singles
        subplot(3,1,1)
        if doNormalize == 1
            bar( sCounts/totSingles)
            ylim([0 1])
            ylabel('Relative singles')
        else
            bar( sCounts)
            ylabel('Singles (Hz)')
        end
        title( [strDist, ' ', type, ' ', num2str(ii)])
        text( 0.05, 0.91, ['Total counts = ', num2str( round(totSingles/1000)), ' kHz'], 'units', 'normalized')
        set( gca, 'xtick', 1:length(sCounts), 'xticklabels', singleNames)
        box on
        
        % now coincidences
        subplot(3,1,2)
        if doNormalize == 1
            bar( cCounts/totCoin)
            ylabel('Relative coincidences')
        else
            bar( cCounts)
            ylabel('Coincidences (Hz)')
        end
        
        box on
        ylim([0 ceil(max(ylim)*10)/10])
        xlim([0 max( nCoinName, nSingleName)+1])
        text( 0.05, 0.91, ['Total counts = ', num2str( round(totCoin,2)), ' Hz'], 'units', 'normalized')
        xlabel('Output Channel')
        set( gca, 'xticklabels', [])
        
        
        % plot biased probability
        subplot(3,1,3)
        hold on
        bar( -Praw./sum(Praw)); %bar( -Praw/totCoin)
        bar( Pcor./sum(Pcor)); %bar( Pcor/totCoin)
        hold off
        ylabel('Probability')
        xlabel('Output State')
        set( gca, 'xtick', 1:nStates, 'xticklabels', strState,...
            'ytick', [-1 0 1], 'yticklabels', {'1', '0', '1'})
        legend('Uncorrected', 'Corrected')
        ylim([-1 1])
        box on
        end
        
    end
end


%% Make plot 
% data 
P_pump = zeros(1,6);
Power = [10 20 40 60 80 100];
for ii = 1:6
    store = Pcor_store{ii}.(strDist)./sum(Pcor_store{ii}.(strDist));
    P_pump(ii) = store(6);
end

figure(2)
weightfit = [1 1 1 1 1 1];
fit = fmlsqAB(Power,P_pump,weightfit);
A = fit(1) %slope
B = fit(3) %intercept
hold on
yline(1,':','color','red','linewidth',3);
plot(Power,P_pump,'*','color','blue','MarkerSize',16,'linewidth',3);
plot([0 100],[A*0+B A*100+B],'--','color','black','linewidth',3);
ylim([0 1.05]);
xlim([0 100]);
%grid on
set(gca,'Fontsize',16);
xlabel('Pump power [mW]','Interpreter','latex');
ylabel('Probability $|1 1 1 0>$ state','Interpreter','latex');
%title('Identity matrix','interpreter','latex');
legend({'Theory','Experiment','Linear fit'},'Interpreter','latex','location','southeast');
set(gca,'TickLabelInterpreter','latex')



    

%% function space
function states = getStates( n, m)
% computes all the state vectors for a n photons in a m mode interferomter
% output:   
%   states      a list of all state vectors

% all possible number of states is a multiset
nStates = nchoosek( n+m-1,n);

% allocate memory
states = zeros( nStates, m);
% d(s)_j = Omega(j,:) output mode assingment list
modes_list = nmultichoosek(1:m, n);   

% loop through all possibilities
for ii = 1 : nStates
    list = modes_list(ii,:);
    for jj = 1 : n
        states(ii,list(jj)) = states(ii,list(jj)) + 1;
    end
end
end

function strState = getStateString( states)
% converts the matrix of states into a structure of strings
% this is used as xticklabel when you plot the distribution as function of
% all the output state vectors

[nStates, nModes] = size(states);
for ii = 1 : nStates
    strState{ii} = [];
    for jj = 1 : nModes
        strState{ii} = [strState{ii}, num2str(states(ii,jj))];
    end
end
end

function [sCounts, totSingles, singleNames] = getSingles( Singles, nTrigger)
% extract the single counts
% input:
%   Singles         a struct with all the singles
% output:
%   sCounts         array with average counts in each channel
%   totSingles      the total number of counts
%   singleNames     structure with strings of each channelnames

% extract singles
singleNames = fieldnames( Singles);
nSingleName = length( singleNames);

sCounts = zeros( 1, nSingleName);
for jj = 1 : nSingleName - nTrigger
    sCounts(jj) = mean( Singles.(singleNames{jj}));
end
totSingles = sum( sCounts);
end

function [cCounts, totCoin, coinNames] = getCoincidences(Coincidences)
% extract the single counts
% input:
%   Coincidences    a struct with all the coincidences
% output:
%   cCounts         array with average counts in each channel
%   totCoin         the total number of coincidences counts
%   coinNames       structure with strings of each channelnames

% extract coincidences
coinNames = fieldnames( Coincidences);
nCoinName = length(coinNames);

cCounts = zeros( 1, nCoinName);
for jj = 1 : nCoinName
    cCounts(jj) = mean( Coincidences.(coinNames{jj})); % or sum? mwah.. 
end
totCoin = sum( cCounts);
end

function weightfactor = getWeightfactor(Folder, FileName)
% extract the weightfactors for the fiber splitter
A = load([Folder,FileName]);
weightfactor = A.W;
end



function [Praw, Pcor] = getProbabilities( Coincidences, weightfactor, states, nStates, nPhotons, trigCh, nModes)
% get the biased (Praw) and unbiased (Pcorr) probabilities associated with
% each output state. 
% Input:
%   Coincidences    struct with all coincidence data
%   weightfactor    array with the relative weights of each detector
%                   (used to correct for biased sampling)
%   states          list of all the output states
%   nStates         total number of output states
%   nPhotons        total number of photons in experiment (excl. herald)
%   trigCh          the swabian channel of the trigger/herald photon
% output:
%   Praw            Probability uncorrected for the biased sampling
%   Pcor            Probability   corrected for the biased sampling

% allocate memory
Praw = zeros(1,nStates);
Pcor = zeros(1,nStates);

%loop through all the states
for nn = 1 : nStates
    
    % get a list of all channel combinations associated with the state
    list = stateToChannel( states(nn,:));
  
    for mm = 1 : size(list,1)
        % build up the string of the fieldname so we can extract the data
        str = '';
        for kk = 1 : nPhotons
            str = [str, 'C', num2str(list(mm,kk))];
        end
        str = [str, 'C', num2str(trigCh)];
        
        Praw(nn) = Praw(nn) + mean(Coincidences.(str));
    end
   
end

correction = getCorrection(states, nModes, weightfactor);
correction = correction';  %column vector to row vector
Pcor = Praw./correction;

end

function correction = getCorrection(states, nModes, weightfactor)

% hardcoded the channels which belong to a certain output mode
splitter1 = [9 10 11];
splitter2 = [6 7 12];
splitter3 = [5 8 14];
splitter4 = [1 2 4];

splitterlist = [splitter1; splitter2; splitter3; splitter4];

correction = ones(size(states,1),1);
for ii = 1:size(states,1)
	for jj = 1:nModes
        if states(ii,jj) == 0
            correction(ii) = correction(ii)*1;
        else
            list = nchoosek(splitterlist(jj,:),states(ii,jj));
            Pk_nMode = 0;
            for kk = 1:size(list,1)
                Pk_nMode = Pk_nMode + factorial(states(ii,jj))*prod(weightfactor(list(kk,:)));
            end
                correction(ii) = correction(ii)*Pk_nMode;
        end
        
    end

end

% At this point, we have corrected for multinomial distribution introduced
% by fiber beam splitters (see Feito et al.)

end

function list = stateToChannel( state)
% converts a state into the possible combinations of channels of the
% Swabian. The state should contain max 3 photons over 4 modes
% Output:
%   List    matrix of all channel combinations which belong to this state
%           each row is a combination of channels

% hardcoded the channels which belong to a certain output mode
splitter1 = [9 10 11];
splitter2 = [6 7 12];
splitter3 = [5 8 14];
splitter4 = [1 2 4];

% sanity check:
if sum( state) ~= 3
    state
    warning('total number of photons ~= 3')
end

list = [];
% loop through the first mode
a = nchoosek( splitter1, state(1));
for ii = 1 : size(a,1)
    % 2nd mode
    b = nchoosek(splitter2, state(2));
    for jj = 1: size(b,1)
        % 3rd mode
        c = nchoosek(splitter3, state(3));
        for kk = 1 : size(c,1)
            % 4th mode
            d = nchoosek( splitter4, state(4));
            for ll = 1 : size(d,1)
                list = [list; a(ii,:),b(jj,:),c(kk,:),d(ll,:)];
            end
        end
    end
end
% sort the list to ascending channel number
list = sort( list,2);
end

function Pk_exp = getPk_Experiment(list_states, n, mode0)
% Gets Pk for the experimental data, i.e., the distribution of photons
% in one output mode of the chip
% input:   
%   data            a list of all states and the rate
%                   Format:
%                   Rate    state (1x4 array), e.g. for the |1110> state:
%                   0.3 1 1 1 0
%   n               number of photons
%   mode0           the mode of interest (e.g., 1 or 2)
%   Output:
%   Pk_exp          Pk belonging of the state of interest

% allocate memory
Rk_experimental = zeros(1,n+1);

% run through all possible number of photons
for k = 0:1:n
    
    % loop through all possibles states
    for ii = 1:length(list_states)
        
        % something??
        if list_states(ii,mode0+1) == k  
            Rk_experimental(k+1) = Rk_experimental(k+1) + list_states(ii,1);
        end
    end
end

% normalize data
R_tot = sum(Rk_experimental);
Pk_exp = Rk_experimental./R_tot;
end




function plotPk( Pk_data, nPhotons, nFiles, Modes, Canonical)
%timesteps = [.1 .2 .5 5 2 1]; % volgorde?
timesteps = [.1 .2 .5 1 2 5];

% SIMULATION RESULTS
mode_of_interest = Modes;

% Set up storage
P_ind_num = zeros(length(timesteps),nPhotons+1);
P_dist_num = zeros(length(timesteps),nPhotons+1);

% First, indistinguishable 
x = 1;
for jj = 1:length(timesteps)
    %P_ind_num(jj,:) = [1 0 0 0];
    P_ind_num(jj,:) = getNumericalHubbard(nPhotons,4,timesteps(jj),mode_of_interest,x); 
end

% Second, distinguishable
x = 0;
for jj = 1:length(timesteps)
    P_dist_num(jj,:) = getNumericalHubbard(nPhotons,4,timesteps(jj),mode_of_interest,x); 
    %P_dist_num(jj,:) = [1 0 0 0];
end

for ii = Modes
    figure()
    clf
    
    % make a subplot without white space between them
    %[ha, pos] = tight_subplot(1,6, 0, [.05 0.55], [.1 .1]); %[.15 0.1], [.1 .1]
    [ha, pos] = tight_subplot(2,3, 0, [.15 .15], [.1 0.20]); %[.15 0.1], [.1 .1]
    FancyPlot( get( gcf, 'Number'), 0)
    set( gca, 'FontSize', 10)
    
    for jj = 1 : nFiles
        % select the frame of the subplot
        axes(ha(jj))
        
        % plotting
        hold on
%         bar(  0 : nPhotons, Canonical, 'color', [90 90 90]/255)  
        bar(  0 : nPhotons, Canonical, 'FaceColor', [200 200 200]/255, 'EdgeColor', [200 200 200]/255 )  
%         plot( 0 : nPhotons, Pk_data{jj}.Indistinguishable(ii,:)', 'o','color','blue','MarkerFaceColor','blue','MarkerSize',8,'LineWidth',1.5)
plot( 0 : nPhotons, Pk_data{jj}.Indistintuighable(ii,:)', 'o','color','blue','MarkerFaceColor','blue','MarkerSize',8,'LineWidth',1.5)
        plot( 0 : nPhotons, Pk_data{jj}.Distinguishable(ii,:)', 's','color','blue','MarkerFaceColor','blue','MarkerSize',8,'LineWidth',1.5)
        plot( 0 : nPhotons, P_ind_num(jj,:), 'o','color','black','MarkerSize',8,'LineWidth',1.5)
        plot( 0 : nPhotons, P_dist_num(jj,:), 's','color','black','MarkerSize',8,'LineWidth',1.5)
        hold off
        
        % setup the axes
        set( gca, 'xtick', 0:nPhotons, 'ytick', [0 .5 1])
        xlim([-.5 nPhotons+.5])
        ylim([0 1.1])
        box on
        set(gca, 'Fontsize',16)

        % setup the time title
        text( .8, 1.03, ['$\tau$ = ', num2str(timesteps(jj),'%3.1f')],'Fontsize',16,'Interpreter','Latex')

        % set up xticks at the bottom panels
        if jj > nFiles/2
            set( gca, 'xticklabels', {'0', '1', '2', '3'})
            if jj == 5
                xlabel('Photon number $k$','Interpreter','Latex')
                set(gca, 'Fontsize',16)
            end
        end

        % set up yticks at the left panels
        if mod(jj,nFiles/2) == 1
            set( gca, 'yticklabels', {'0', '0.5', '1.0'})
            ylabel('Probability $P(k)$','Interpreter','Latex')
            set(gca, 'Fontsize',16)
        end
        
        % setup the legend
        if jj == nFiles
            h = legend({'Canonical', 'Indistinguishable (Exp.)','Distinguishable (Exp.)','Indistinguishable (Num.)','Distinguishable (Num.)'},'Interpreter','Latex')
            set(gca, 'Fontsize',16)
            pos = get(h,'position');
            set(h, 'position',[0.78 0.5 pos(3:4)])
        end
        
    end
end

end

function combs = nmultichoosek(values, k)
    %// Return number of multisubsets or actual multisubsets.
    if numel(values)==1 
        n = values;
        combs = nchoosek(n+k-1,k);
    else
        n = numel(values);
        combs = bsxfun(@minus, nchoosek(1:n+k-1,k), 0:k-1);
        combs = reshape(values(combs),[],k);
    end
end

function P_Canonical = CanonicalState(nModes,nPhotons)
    M = nModes;
    N = nPhotons;
    for k = 0:1:N
        P_Canonical(k+1) = nchoosek(M+N-k-2,N-k)/nchoosek(M+N-1,N);
    end
    
end

%% Function for numerical simulation
function [Pk, P_microstate] = getNumericalHubbard(N,M,tau,mode_of_interest,x)
% Function that does numerical simulation for Bose-Hubbard time evolution
% N = number of photons
% M = number of modes
% tau = time
% mode_of_interest is output channel for which we calculate the output mode
% distribution
% x = distinguishability parameter. x = 0 for distinguishable, x = 1 for
% indistinguishable

    % Setup Hubbard-Hamiltonian
    a = 1;
    b = 1;
    c = 1;
    H = sparse(diag(a*ones(1,M))+ diag(b*ones(1,M-1),1) +diag(c*ones(1,M-1),-1));
    H(1,M) = 1;
    H(M,1) = 1;
    H = H - speye(M);
    U_Hubbard = expm(-1i*H); 


    % Distinguishability matrix S:
    S = ones(M)*x;
    S = S + eye(M).*diag(1-S);

    % Input state r-vector
    input = [1:N];      % d(r)_j = input mode assignment list

    % Microstates (sample output states)
    Omega = nmultichoosek(1:M, N);   % d(s)_j = Omega(j,:) output mode assingment list

    % Storage  - all microstates s | P(s) Haar 1 | P(s) Haar 2 | .. 
    microstate_store = zeros(length(Omega),1); % 
    Pk_store = zeros(1,N+1); 


        % Time evolution
        U = U_Hubbard^tau;

        % Makes sure in line with convention of Tichy, 2015 - Actually
        % doens't really matter
        U = transpose(U);  

        % Matrix for all s with columns P[s] | s1 | s2 | .. | sM
        microstate_result = Simulation_microstates(input,U,Omega,N,M,S); 

        % P[k] for mode of interest with columns P(0) | P(1) | .. | P(N)
        Pk_result = Simulation_Pk(microstate_result,N,mode_of_interest);

        % results
        P_microstate = microstate_result(:,1);
        Pk = Pk_result;


    %% Relevant functions
    % Calculate number distribution via modified A&A algorithm
    function microstate_list = Simulation_microstates(input,U,Omega,N,M,S)

            microstate_list = zeros(length(Omega),M+1); % probability | mode 1 | mode 2 | mode 3 | mode 4
            subU = U(input,:);
            subM = zeros(N,N);

            for ss = 1:length(Omega)

                d = Omega(ss,:); % output mode assignment list of interest
                for ii = 1:length(d)
                    subM(:,ii) = subU(:,d(ii));
                end

                % create output state s (convert mode assignment list)
                sample = zeros(1,M);
                for ii = 1:length(d)
                    sample(d(ii)) = sample(d(ii)) + 1;
                end

                Norm = 1/prod(factorial(sample));   % Normalization factor

                % Do multidimensional permanent calculation
                % Create permanent

                W = [];
                for kk = 1:N
                    for ll = 1:N
                        for jj = 1:N
                            W(kk,ll,jj) = subM(kk,jj)* conj(subM(ll,jj))*S(ll,kk);
                        end
                   end
                end

                P_s = Norm*abs(permT(W));       % permT function from Matthijs

                % storage
                microstate_list(ss,1) = P_s;
                microstate_list(ss,2:end) = sample;
            end

    end

    function Pk_simulation = Simulation_Pk(microstate_list,N,mode_of_interest)

        Pk_simulation = zeros(1,N+1);
        for k = 0:1:3
            for ii = 1:length(microstate_list)
                if microstate_list(ii,mode_of_interest+1) == k 
                    Pk_simulation(k+1) = Pk_simulation(k+1) + microstate_list(ii,1);
                end
            end
        end


    end

end