clear all;
close all;
clc;

%% Import weightfactors
% Add folder in which weightfactors are stored
addpath('C:\Users\frank\Documents\MasterAssignment\Experimental_Processing\20210313_Testing_Haar_Distinguishable\20210312_weightfactors');

w = load('20210317_Thermalization_weightfactor_FBS_Characterization.mat');


%% Import measurement results Haar matrices
% Some settings
nTrigger = 1;
trigCh = 13;
doNormalize = 1;
doPlot = 0;
nPhotons = 3;
nModes = 4;
str.Dist = 'Distinguishable';

% Some state preparation
% identify all state vectors
states = getStates( nPhotons, nModes);
strState = getStateString( states);
nStates= size( states,1);             % Got it


% Add folder
Folder = 'C:\Users\frank\Documents\MasterAssignment\Experimental_Processing\20210317_Blindness\';
name = ['20210317_Thermalization_data_Distinguishable_Identity_*.mat'];
    
% identify all data files
files = dir([Folder, name]);
nFiles = length(files);
    
% Setup space for microstate probability
P_microstate = zeros(nFiles,nStates);
R_total_events = zeros(nFiles,1);
Pk_store = cell(nFiles,1);


% loop through all the files
for ii = 1 : nFiles
    % load in the data
    data{ii} = load( [Folder, files(ii).name]) % Files is in wrong order!!! 
    data{ii}.data.Coincidences
 	size(data{ii}.data.Coincidences)

    % Derive microstate probabilities
%     [Praw, Pcor] = getProbabilities( data{ii}.data.Coincidences, w, states, nStates, nPhotons, trigCh, nModes);
%     
%     % Save result
%     P_microstate(ii,:) = Pcor./sum(Pcor); % Actually rate, not probability yet
%     R_total_events(ii,1) = sum(Praw); % Hz, actually measured rates (uncorrected)
%      
%     for jj = 1 : nModes
%         Pk_store{ii}.(['mode_',num2str(jj)])(1,:) = getPk_Experiment([Pcor',states], nPhotons, jj);
%     end
%     
    
end

%% Save data
% Where to save data
Folder = 'C:\Users\frank\Documents\MasterAssignment\Experimental_Processing\20210317_Blindness\';
nameStr = 'Distinguishable_Identity_microstate_probabilities.mat';
save( [Folder, nameStr], 'P_microstate');

nameStr = 'Distinguishable_Identity_Pk.mat';
save( [Folder, nameStr], 'Pk_store');

%% Functions


function [cCounts, totCoin, coinNames] = getCoincidences(Coincidences)
% extract the single counts
% input:
%   Coincidences    a struct with all the coincidences
% output:
%   cCounts         array with average counts in each channel
%   totCoin         the total number of coincidences counts
%   coinNames       structure with strings of each channelnames

% extract coincidences
coinNames = fieldnames( Coincidences);
nCoinName = length(coinNames);

cCounts = zeros( 1, nCoinName);
for jj = 1 : nCoinName
    cCounts(jj) = mean( Coincidences.(coinNames{jj}));
end
totCoin = sum( cCounts);
end

function [Praw, Pcor] = getProbabilities( Coincidences, weightfactor, states, nStates, nPhotons, trigCh, nModes)
% get the biased (Praw) and unbiased (Pcorr) probabilities associated with
% each output state. 
% Input:
%   Coincidences    struct with all coincidence data
%   weightfactor    array with the relative weights of each detector
%                   (used to correct for biased sampling)
%   states          list of all the output states
%   nStates         total number of output states
%   nPhotons        total number of photons in experiment (excl. herald)
%   trigCh          the swabian channel of the trigger/herald photon
% output:
%   Praw            Probability uncorrected for the biased sampling
%   Pcor            Probability   corrected for the biased sampling

% allocate memory
Praw = zeros(1,nStates);
Pcor = zeros(1,nStates);

%loop through all the states
for nn = 1 : nStates
    
    % get a list of all channel combinations associated with the state
    list = stateToChannel( states(nn,:));
  
    for mm = 1 : size(list,1)
        % build up the string of the fieldname so we can extract the data
        str = '';
        for kk = 1 : nPhotons
            str = [str, 'C', num2str(list(mm,kk))];
        end
        str = [str, 'C', num2str(trigCh)];
        
        % now add the probabilities up (still biased!)
        Praw(nn) = Praw(nn) + Coincidences.(str);
    end
   
end

correction = getCorrection(states, nModes, weightfactor);
correction = correction';  %column vector to row vector
Pcor = Praw./correction;

end

function correction = getCorrection(states, nModes, weightfactor)

% hardcoded the channels which belong to a certain output mode
splitter1 = [9 10 11];
splitter2 = [6 7 12];
splitter3 = [5 8 14];
splitter4 = [1 2 4];

splitterlist = [splitter1; splitter2; splitter3; splitter4];

correction = ones(size(states,1),1);
for ii = 1:size(states,1)
	for jj = 1:nModes
        if states(ii,jj) == 0
            correction(ii) = correction(ii)*1;
        else
            list = nchoosek(splitterlist(jj,:),states(ii,jj));
            Pk_nMode = 0;
            for kk = 1:size(list,1)
                Pk_nMode = Pk_nMode + factorial(states(ii,jj))*prod(weightfactor(list(kk,:)));
            end
                correction(ii) = correction(ii)*Pk_nMode;
        end
        
    end

end

% At this point, we have corrected for multinomial distribution introduced
% by fiber beam splitters (see Feito et al.)

end
        
function states = getStates( n, m)
% computes all the state vectors for a n photons in a m mode interferomter
% output:   
%   states      a list of all state vectors

% all possible number of states is a multiset
nStates = nchoosek( n+m-1,n);

% allocate memory
states = zeros( nStates, m);
% d(s)_j = Omega(j,:) output mode assingment list
modes_list = nmultichoosek(1:m, n);   

% loop through all possibilities
for ii = 1 : nStates
    list = modes_list(ii,:);
    for jj = 1 : n
        states(ii,list(jj)) = states(ii,list(jj)) + 1;
    end
end
end

function strState = getStateString( states)
% converts the matrix of states into a structure of strings
% this is used as xticklabel when you plot the distribution as function of
% all the output state vectors

[nStates, nModes] = size(states);
for ii = 1 : nStates
    strState{ii} = [];
    for jj = 1 : nModes
        strState{ii} = [strState{ii}, num2str(states(ii,jj))];
    end
end
end

function list = stateToChannel( state)
% converts a state into the possible combinations of channels of the
% Swabian. The state should contain max 3 photons over 4 modes
% Output:
%   List    matrix of all channel combinations which belong to this state
%           each row is a combination of channels

% hardcoded the channels which belong to a certain output mode
splitter1 = [9 10 11];
splitter2 = [6 7 12];
splitter3 = [5 8 14];
splitter4 = [1 2 4];

% sanity check:
if sum( state) ~= 3
    state
    warning('total number of photons ~= 3')
end

list = [];
% loop through the first mode
a = nchoosek( splitter1, state(1));
for ii = 1 : size(a,1)
    % 2nd mode
    b = nchoosek(splitter2, state(2));
    for jj = 1: size(b,1)
        % 3rd mode
        c = nchoosek(splitter3, state(3));
        for kk = 1 : size(c,1)
            % 4th mode
            d = nchoosek( splitter4, state(4));
            for ll = 1 : size(d,1)
                list = [list; a(ii,:),b(jj,:),c(kk,:),d(ll,:)];
            end
        end
    end
end
% sort the list to ascending channel number
list = sort( list,2);
end

function combs = nmultichoosek(values, k)
    %// Return number of multisubsets or actual multisubsets.
    if numel(values)==1 
        n = values;
        combs = nchoosek(n+k-1,k);
    else
        n = numel(values);
        combs = bsxfun(@minus, nchoosek(1:n+k-1,k), 0:k-1);
        combs = reshape(values(combs),[],k);
    end
end

function P_Canonical = CanonicalState(nModes,nPhotons)
    M = nModes;
    N = nPhotons;
    for k = 0:1:N
        P_Canonical(k+1) = nchoosek(M+N-k-2,N-k)/nchoosek(M+N-1,N);
    end
    
end


function Pk_exp = getPk_Experiment(list_states, n, mode0)
% Gets Pk for the experimental data, i.e., the distribution of photons
% in one output mode of the chip
% input:   
%   data            a list of all states and the rate
%                   Format:
%                   Rate    state (1x4 array), e.g. for the |1110> state:
%                   0.3 1 1 1 0
%   n               number of photons
%   mode0           the mode of interest (e.g., 1 or 2)
%   Output:
%   Pk_exp          Pk belonging of the state of interest

% allocate memory
Rk_experimental = zeros(1,n+1);

% run through all possible number of photons
for k = 0:1:n
    
    % loop through all possibles states
    for ii = 1:length(list_states)
        
        % something??
        if list_states(ii,mode0+1) == k  
            Rk_experimental(k+1) = Rk_experimental(k+1) + list_states(ii,1);
        end
    end
end

% normalize data
R_tot = sum(Rk_experimental);
Pk_exp = Rk_experimental./R_tot;
end
