% process the fiber splitter data
clc
close all
clear 


% settings
nTrigger = 1;
trigCh = 13;
doNormalize = 1;
doPlot = 0;
nPhotons = 3;
nModes = 4;

% Add folders

% file selection
Folder0 = [pwd,'\20211119_(In)Distinguishable_Haar+BH\'];
nameData = '2021112*_data_Thermalization_data_';

% Type selection
% type = 'BoseHubbard';
type = 'Haar';


% identify all state vectors
states = getStates( nPhotons, nModes);
strState = getStateString( states);
nStates= size( states,1);


% where to save the data
FolderS = [pwd,'\20211119_(In)Distinguishable_Haar+BH\Results\'];


% get the weightfactors to correct against biased sampling
fiberFolder = [pwd,'\20211119_(In)Distinguishable_Haar+BH\FiberSplitter\'];
fiberName = '202111*_Weightfactor_Fibersplitter_batch';


% HOM visibilities
HOM = [.89 .75 .75];
% HOM1  input mode 2 - 3
% HOM2  input mode 1 - 3
% HOM3  input mode 1 - 2


% the names of the different types of measurements
name{1} = 'Indistinguishable';
name{2} = 'Distinguishable';
sName{1} = 'Indistinguishable';
sName{2} = 'Distinguishable';



% setup the parameters for BoseHubbard or Haar
if strcmp(upper(type), 'BOSEHUBBARD')
    Folder = [Folder0, 'Bose-Hubbard\'];
    timeRange = 1:6;
elseif strcmp(upper(type), 'HAAR')
    Folder = [Folder0, 'Haar\'];
    timeRange = 1 : 120;
end


% define experimental parameters
nBatches = 3;
nTimeSteps = length(timeRange);
nOutputs = 220;
nStates  = size( states,1);


% loop through the data and process it
for nn = 1 : length(name)
    
      
    % loop through all batches
    % each batch is a short measurement which is repeated nBatches times
    % hence I should collect al data and append it before processing
    cCounts{ nBatches} = zeros( nTimeSteps, nOutputs);
    Pcor.(sName{nn}){ nTimeSteps} = zeros( nBatches, nStates);
    Praw.(sName{nn}){ nTimeSteps} = zeros( nBatches, nStates);

    for bb = 1 : nBatches
        
        
        % load in the weight factors
        weightfactor(bb,:) = getWeightfactor(fiberFolder,...
            [fiberName, num2str(bb),'.mat']);
        
        
        clear data
        % loop through all timesteps in this batch
        ind = 1;
        for ii = timeRange
            
            
            % load in the data
            filename = [nameData, name{nn}, '*_', num2str(ii),...
                '_batch_', num2str(bb),'.mat'];
            files = dir([Folder, filename]);
            if length(files) > 1
                warning('Multiple datasets found, only take the 1st one')
            elseif length(files) == 0
                warning(['No files found, skip this instance (', name{nn}, ', batch ',num2str(bb), ', timestep ', num2str(ii), ')'])
                ind = ind+1;
                continue  
            end
            data{ind} = load( [Folder, files(1).name]);
            
            
            % get the singles
            [sCounts, totSingles, singleNames] = getSingles(...
                data{ind}.data.Singles, nTrigger);
            nSingleName = length( singleNames);
            
            
            % get the coincidences
            [cCounts{ind}(bb,:), totCoin, coinNames] = getCoincidences(...
                data{ind}.data.Coincidences, data{ind}.data.IntegrationTime);
            nCoinName = length( coinNames);
            

            % unnormalized probabilities to state
            [Praw.(sName{nn}){ind}(bb,:), Pcor.(sName{nn}){ind}(bb,:)]...
                = getProbabilities( data{ind}.data.Coincidences,...
                    weightfactor(bb,:), states, nStates, nPhotons,...
                    trigCh, nModes, data{ind}.data.IntegrationTime);
            
            
            
            % start plotting the data
            if doPlot == 1
                plotSingles( sCounts, totSingles, doNormalize, singleNames)
                plotCoincidences( cCounts, totCoin, nCoinName,...
                    nSingleName, Praw, Pcor, nStates, strState)
            end
            ind = ind+1;
            
        end
    end
    
    % now combine all counts / Pcorr from all batches and compute Pk
    % loop through all time steps
    for ii = 1 : nTimeSteps
        
        % normalize Pcor
        if size( Pcor.(sName{nn}){ii},1)>1
            temp = sum( Pcor.(sName{nn}){ii});
        else
            temp = Pcor.(sName{nn}){ii};
        end
        PcorN.(sName{nn}){ii} = temp / sum(temp);
        
        % get Pk distribution for each mode
        Pk.(sName{nn}){ii} = zeros(nModes,nModes);
        
            % loop through all modes
        for jj = 1 :nModes
            % only do it if there is actually some data
            if max( max( PcorN.(sName{nn}){ii} )) >0
                Pk.(sName{nn}){ii}(jj,:) = getPk_Experiment([PcorN.(sName{nn}){ii}',states], nPhotons, jj);
            end
        end
    end
end


%% Total Variation distance plot

f1 = figure();
f1.Position = [680,571,630,527];

Distances = GetL1Dist(Pk, 3, 'Canonical', type);

D_av  =  mean(Distances,3);
D_max =  max(Distances,[],3);
D_min =  min(Distances,[],3);
D_std =  std(Distances,0,3);

writeFile = 0;

if writeFile == 1
    if strcmp(upper(type), 'HAAR')
        f_id = fopen('valuesTVDHaar.txt','w');
        fprintf(f_id, 'Indistinguishable\n')
        fprintf(f_id, 'Mean \tStd \tMax \tMin\n');
        fprintf(f_id, '%4.3f\t%4.3f\t%4.3f\t%4.3f\n',...
            [D_av(1,:); D_std(1,:); D_max(1,:); D_min(1,:)]);
        
        
        fprintf(f_id, '\nDistinguishable\n')
        fprintf(f_id, 'Mean \tStd \tMax \tMin\n');
        fprintf(f_id, '%4.3f\t%4.3f\t%4.3f\t%4.3f\n',...
            [D_av(2,:); D_std(2,:); D_max(2,:); D_min(2,:)]);
        fclose(f_id);
    end
    
    
    if strcmp(upper(type), 'BOSEHUBBARD')
        f_id = fopen('valuesTVDBoseHubbard.txt','w');
        fprintf(f_id, 'Indistinguishable\n')
        fprintf(f_id, 'Mean \n');
        fprintf(f_id, '%4.3f\n',...
            [D_av(1,:)]);
        
        
        fprintf(f_id, '\nDistinguishable\n')
        fprintf(f_id, 'Mean \n');
        fprintf(f_id, '%4.3f\n',...
            [D_av(2,:)]);
        fclose(f_id);
    end
    
end

offset = 0.05;

% make plots
hold on
box on
timesteps = [0.1 0.2 0.5 1 2 5];


% colors

c1 = [0.929, 0.192, 0.192]; %red edge
c2 = [0.125, 0.470, 0.772]; %blue edge

blue_face = [0.6314    0.8039    1.0000];
red_face = [1.0000    0.6392    0.6392];

% average

if strcmp(upper(type), 'HAAR')
eb1 = errorbar(timesteps + offset*timesteps, D_av(1,:), D_std(1,:), 'o',...
               'LineWidth', 1.5, 'Color', c1);
eb1.MarkerFaceColor = red_face;
eb2 = errorbar(timesteps - offset*timesteps, D_av(2,:), D_std(2,:), 'o',...
               'LineWidth', 1.5, 'Color', c2 );
eb2.MarkerFaceColor = blue_face;    
end

if strcmp(upper(type), 'BOSEHUBBARD')
eb1 = plot(timesteps + offset*timesteps, D_av(1,:), 'o',...
               'LineWidth', 2, 'MarkerSize', 8, 'Color', c1);
eb1.MarkerFaceColor = red_face;
eb2 = plot(timesteps - offset*timesteps, D_av(2,:), 'o',...
               'LineWidth', 2, 'MarkerSize', 8, 'Color', c2);
eb2.MarkerFaceColor = blue_face;
    
end



if strcmp(upper(type), 'HAAR') 
% max
plot(timesteps + offset*timesteps, D_max(1,:),'^' , 'color', c1, 'LineWidth', 1.3, 'MarkerFaceColor', red_face)
plot(timesteps - offset*timesteps, D_max(2,:),'^', 'color', c2, 'LineWidth', 1.3, 'MarkerFaceColor', blue_face)


% min
plot(timesteps + offset*timesteps, D_min(1,:),'v', 'color', c1, 'LineWidth', 1.3, 'MarkerFaceColor', red_face)
plot(timesteps - offset*timesteps, D_min(2,:),'v', 'color', c2, 'LineWidth', 1.3, 'MarkerFaceColor', blue_face)
end

% aesthetic plot options

% legend(' Indistinguishable  Exp(   ) | max/min(     )', ' Distinguishable  Exp(   ) | max/min(     )', ' ', ' ', ' ', ' ')
legend(' Indistinguishable  Exp(   )', ' Distinguishable  Exp(   )')


xlabel('Normalized time    ','interpreter','TeX')
set(gca, 'XScale', 'log')
set(gca, 'LineWidth', 1.2)
set(gca, 'XTick', [0.1 0.2 0.5 1 2 5], 'FontSize',15)

ylabel('Total Variation Distance')

xlim([0.07 6])
ylim([0 0.75])
hold off
  


  


%% function space
function states = getStates( n, m)
% computes all the state vectors for a n photons in a m mode interferomter
% output:   
%   states      a list of all state vectors

% all possible number of states is a multiset
nStates = nchoosek( n+m-1,n);

% allocate memory
states = zeros( nStates, m);
modes_list = nmultichoosek(1:m, n);   

% loop through all possibilies
for ii = 1 : nStates
    list = modes_list(ii,:);
    for jj = 1 : n
        states(ii,list(jj)) = states(ii,list(jj)) + 1;
    end
end
end

function strState = getStateString( states)
% converts the matrix of states into a structure of strings
% this is used as xticklabel when you plot the distribution as function of
% all the output state vectors

[nStates, nModes] = size(states);
for ii = 1 : nStates
    strState{ii} = [];
    for jj = 1 : nModes
        strState{ii} = [strState{ii}, num2str(states(ii,jj))];
    end
end
end

function list = stateToChannel( state)
% converts a state into the possible combinations of channels of the
% Swabian. The state should contain max 3 photons over 4 modes
% Output:
%   List    matrix of all channel combinations which belong to this state
%           each row is a combination of channels

% hardcoded the channels which belong to a certain output mode
splitter1 = [9 10 11];
splitter2 = [6 7 12];
splitter3 = [5 8 14];
splitter4 = [1 2 4];

% sanity check:
if sum( state) ~= 3
    state
    warning('total number of photons ~= 3')
end

list = [];
% loop through the first mode
a = nchoosek( splitter1, state(1));
for ii = 1 : size(a,1)
    % 2nd mode
    b = nchoosek(splitter2, state(2));
    for jj = 1: size(b,1)
        % 3rd mode
        c = nchoosek(splitter3, state(3));
        for kk = 1 : size(c,1)
            % 4th mode
            d = nchoosek( splitter4, state(4));
            for ll = 1 : size(d,1)
                list = [list; a(ii,:),b(jj,:),c(kk,:),d(ll,:)];
            end
        end
    end
end
% sort the list to ascending channel number
list = sort( list,2);
end

function [sCounts, totSingles, singleNames] = getSingles( Singles, nTrigger)
% extract the single counts
% input:
%   Singles         a struct with all the singles
% output:
%   sCounts         array with average counts in each channel
%   totSingles      the total number of counts
%   singleNames     structure with strings of each channelnames

% extract singles
singleNames = fieldnames( Singles);
nSingleName = length( singleNames);

sCounts = zeros( 1, nSingleName);
for jj = 1 : nSingleName - nTrigger
    sCounts(jj) = sum( Singles.(singleNames{jj}));
end
totSingles = sum( sCounts);
end

function [cCounts, totCoin, coinNames] = getCoincidences(Coincidences, intTime)
% extract the single counts
% input:
%   Coincidences    a struct with all the coincidences
%   intTime         the integration time (in seconds)
% output:
%   cCounts         array with total counts in each channel
%   totCoin         the total number of coincidences counts
%   coinNames       structure with strings of each channelnames

if nargin == 1
    intTime = 1;
end

% extract coincidences
coinNames = fieldnames( Coincidences);
nCoinName = length(coinNames);

cCounts = zeros( 1, nCoinName);
for jj = 1 : nCoinName
    cCounts(jj) = sum( Coincidences.(coinNames{jj}))*intTime;
end
totCoin = sum( cCounts);
end

function weightfactor = getWeightfactor(Folder, FileName)
% extract the weightfactors for the fiber splitter
files = dir([Folder, FileName]);

if length(files) > 1
    warning('multiple fiber characterization files found')
end
A = load([Folder,files(1).name]);
weightfactor = A.W;
end

function [Praw, Pcor] = getProbabilities( Coincidences, weightfactor, states, nStates, nPhotons, trigCh, nModes, intTime)
% get the biased (Praw) and unbiased (Pcorr) probabilities associated with
% each output state. 
% Input:
%   Coincidences    struct with all coincidence data
%   weightfactor    array with the relative weights of each detector
%                   (used to correct for biased sampling)
%   states          list of all the output states
%   nStates         total number of output states
%   nPhotons        total number of photons in experiment (excl. herald)
%   trigCh          the swabian channel of the trigger/herald photon
%   nModes          # modes of the chip
%   intTime         Integration time of the measurement, in seconds
% 
% output:
%   Praw            Probability uncorrected for the biased sampling
%   Pcor            Probability   corrected for the biased sampling

% allocate memory
Praw = zeros(1,nStates);
Pcor = zeros(1,nStates);

%loop through all the states
for nn = 1 : nStates
    
    % get a list of all channel combinations associated with the state
    list = stateToChannel( states(nn,:));
  
    for mm = 1 : size(list,1)
        % build up the string of the fieldname so we can extract the data
        str = '';
        for kk = 1 : nPhotons
            str = [str, 'C', num2str(list(mm,kk))];
        end
        str = [str, 'C', num2str(trigCh)];
        
        % now add the probabilities up (still biased!)
        Praw(nn) = Praw(nn) + sum(Coincidences.(str))*intTime;
    end
   
end

% Apply correction against biased sampling
correction = getCorrection(states, nModes, weightfactor);
correction = correction';  %column vector to row vector
Pcor = Praw./correction;

% % Convert from rates to probability
% Praw = Praw / sum(Praw);
Pcor = Pcor / sum(Pcor);
end

function correction = getCorrection(states, nModes, weightfactor)

% hardcoded the channels which belong to a certain output mode
splitter1 = [9 10 11];
splitter2 = [6 7 12];
splitter3 = [5 8 14];
splitter4 = [1 2 4];

splitterlist = [splitter1; splitter2; splitter3; splitter4];

correction = ones(size(states,1),1);
for ii = 1:size(states,1)
	for jj = 1:nModes
        
        if states(ii,jj) == 0
            correction(ii) = correction(ii)*1;
        else
            list = nchoosek(splitterlist(jj,:),states(ii,jj));
            Pk_nMode = 0;
            for kk = 1:size(list,1)
                Pk_nMode = Pk_nMode + factorial(states(ii,jj))*prod(weightfactor(list(kk,:)));
            end
            correction(ii) = correction(ii)*Pk_nMode;
        end
        
    end

end

% At this point, we have corrected for multinomial distribution introduced
% by fiber beam splitters (see Feito et al.)

end

function P_Canonical = CanonicalState(nModes,nPhotons)
% get the Pk distribution for the Canonical state 
    M = nModes;
    N = nPhotons;
    for k = 0:1:N
        P_Canonical(k+1) = nchoosek(M+N-k-2,N-k)/nchoosek(M+N-1,N);
    end
    
end

function [P_Fourier, P] = FourierState( nModes, nPhotons, mode0, threepoint, HOM)
% get the Pk distribution for a Fourier state for outputmode mode0

if nargin == 4
    HOM = ones(1,3);
end
if nargin < 4
    HOM = ones(1,3)
    threepoint = 0;
end

% get the input state & Fourier matrix
input = 1:nPhotons;
% if threepoint correlator == 1: set remaning channels to go straight
if threepoint == 1
    F = eye(nModes);
    F(1:nPhotons,1:nPhotons) = 1 / sqrt(nPhotons) * dftmtx( nPhotons);
else
    F = 1 / sqrt(nModes) * dftmtx( nModes);
end

states = getStates( nPhotons, nModes);
states0= states;
nStates= size( states,1);

temp = [zeros( nStates,1), states];
for ii = 1 : nStates
    
    ind = 1;
    for nn = 1 : nModes
        while states0(ii,nn) > 0
            output(ind) = nn;
            ind = ind+1;
            states0(ii,nn) = states0(ii,nn)-1;
        end
    end
    
    % get the probability
    if min(HOM) == 1
        % ideal case
        P = abs(perm( F(output,input))).^2;
    else
        % with partial distinguishabiliy
        W = buildTensor( F(output,input), HOM(1), HOM(2), HOM(3));
        P = abs(permT(W));
    end
    
    % correct for multiplicity
    Pnorm = P / prod( factorial( states(ii,:)));
    temp(ii,1) = Pnorm;
end

P = temp(:,1);

% get Pk for Fourier
P_Fourier = getPk_Experiment( temp, nPhotons, mode0);
end

function Pk_exp = getPk_Experiment(list_states, n, mode0)
% Gets Pk for the experimental data, i.e., the distribution of photons
% in one output mode of the chip
% input:   
%   data            a list of all states and the rate
%                   Format:
%                   Rate    state (1x4 array), e.g. for the |1110> state:
%                   0.3 1 1 1 0
%   n               number of photons
%   mode0           the mode of interest (e.g., 1 or 2)
%   Output:
%   Pk_exp          Pk belonging of the state of interest

% allocate memory
Rk_experimental = zeros(1,n+1);

% run through all possible number of photons
for k = 0:1:n
    
    % loop through all possibles states
    for ii = 1:length(list_states)
        
        % something??
        if list_states(ii,mode0+1) == k  
            Rk_experimental(k+1) = Rk_experimental(k+1) + list_states(ii,1);
        end
    end
end

% normalize data
R_tot = sum(Rk_experimental);
Pk_exp = Rk_experimental./R_tot;
end


% certification
function [F p1 p2 d1 d2] = Certification( N1, N2, Nt1, Nt2, epsilon)
% Certification as defined in eq. 3.31 of Franks master thesis (p 22)

% get the probabilities
p1 = N1 /Nt1;
p2 = 1 - N2 /Nt2;

% get the bounds
d1 = getErrorBound( p1, N1, epsilon);
d2 = getErrorBound( p2, N2, epsilon);

% get the fidelity
F = p1 -d1 -9/4*(p2+d2);

end

function d = getErrorBound( p, N, epsilon)
% computes the errorbound on p1 and p2 (eq 3.32, p22) of Frank)

d = sqrt( (2*p*(1-p)) / (N * log( 1/sqrt(epsilon))));
end

function [p, d, N, F] = plotCertificationFidelity( Praw,...
                bRange, epsilon, nTimeSteps, sName, nBatches, type, doSave, FolderS)
% plot the lower bounds on the fidelities and returns arrays with all
% values (p, d, and N)
% 
% Input parameters 
%   Praw        A struture with all raw data (uncorrected for detection
%               efficiencies etc.). Data in counts (not normalized!)
%   bRange      An array with the batches you want to check (if > nBatches, 
%               use all data (not just individual batches)
%   epsilon     Probability that the fidelity is correct
%   nTimeSteps  # timesteps (typically 6)
%   sName       A structure with all types (e.g., 'Identity', or 'U1F')
%   nBatches    # batches you measured
%   type        A string stating the type of system you are checking:
%               'BoseHubbard', 'Fourier', 'Haar' or 'Identity'
%   doSave      If 1, make save the plot. Else: don't save
%   FolderS     The path to the folder where you want to save the data
% 
% Output
%   p           A cell array of length 2, p{1} = p1, p{2} = p2
%               Each cell is a matrix where the values are given per batch
%               (sorted by bRange) and the timestep
%   d           A cell array of length 2, d{1} = error bound 1, same as p
%   N
%   F


% in this case we only have the direct 3point Fourier matrix
% this one only processes all batches combined
if length( sName) ==1
    yData = sum(Praw.(sName{1}){1});
    N1  = 0;
    N1t = 1;
    N2  = yData(1) + yData(6) + yData(11) + yData(17) + yData(20);
    N2t = sum(yData);
    [F, p1, p2, d1, d2] = Certification( N1, N2, N1t, N2t, epsilon);
    
    N{1} = 0;
    N{2} = N2t;
    p{1} = 0;
    p{2} = p2;
    d{1} = 0;
    d{2} = d2;

    figure
    subplot(2,1,1)
    plot( 1, p{2}(end,:), '^--', 'color', [.85 .33 .1], 'MarkerSize', 6)
    titleStr = [type, '\_Certification\_Parameters'];
    title( titleStr, 'fontweight', 'normal')
    ylabel('Probability p_2')
    ylim([0 1])
    xlim([.5 1.5])
    set( gca, 'xtick', 1)
    FancyPlot(3)
    
    subplot(2,1,2)
    hold on
    plot( 1, N{2}(end,:), '^--', 'color', [.85 .33 .1], 'MarkerSize', 6)
    xlabel('Timestep')
    ylabel('Counts N_2')
    ylim([0 2.5e4])
    xlim([.5 1.5])
    set( gca, 'xtick', 1)
    FancyPlot(3)
    
    if doSave == 1
        saveStr = [type, '_Certification_Parameters'];
        saveas( gcf, [FolderS, saveStr, '.pdf'])
    end
else

% allocate memory
F = zeros( max(bRange),nTimeSteps);
p{1} = zeros( max(bRange),nTimeSteps);
p{2} = zeros( max(bRange),nTimeSteps);
d{1} = zeros( max(bRange),nTimeSteps);
d{2} = zeros( max(bRange),nTimeSteps);
N{1} = zeros( max(bRange),nTimeSteps);
N{2} = zeros( max(bRange),nTimeSteps);

% go through all selected batches
for bb = bRange
    
    % go through all timesteps
    for ii = 1 : nTimeSteps
        
        % loop through both measurements U^-1 U^-1F
        for nn = 2:3
            
            % if >nBatches: sum up all data
            if bb > nBatches
                yData = sum(Praw.(sName{nn}){ii});
            else
                % yData = Praw.(sName{nn}){ii}(bb,:);
                
                % alternative, include all counts thusfar
                yData = (Praw.(sName{nn}){ii}(1:bb,:));
                if size( yData,1) > 1
                    yData = sum(yData);
                end
            end
            
            % get the counts
            if nn == 2
                N1 = yData(6);
                N1t= sum(yData);
            elseif nn == 3
                N2 = yData(1) + yData(6) + yData(11) + yData(17) + yData(20);
                N2t= sum(yData);
            end
        end
        
        % compute the fidelity for this timestep / batch number
        [F(bb,ii),p1,p2,d1,d2] = Certification( N1, N2, N1t, N2t, epsilon);
        
        N{1}(bb,ii) = N1t;
        N{2}(bb,ii) = N2t;
        p{1}(bb,ii) = p1;
        p{2}(bb,ii) = p2;
        d{1}(bb,ii) = d1;
        d{2}(bb,ii) = d2;
    end
    
    if bb > nBatches
        legEntry{bb} = ['All batches'];
    else
        legEntry{bb} = ['Batch ', num2str(bb)];
    end
    
end

% plot everything
m = ['+','o','*','x','s','d','^','v','>','<','p','h'];

figure
hold on
% loop through all batches
for ii = 1 : max(bRange) 
    
    % give total of all a thicker marker size
    if ii > nBatches
        linWdth = 1.5;
        mrkSize = 8;
    else
        linWdth = 1;
        mrkSize = 6;
    end
    % actually plot
    plot( 1 : nTimeSteps, F(ii,:)', [m(mod(ii-1,length(m))+1)],...
        'MarkerSize', mrkSize, 'linewidth', linWdth)
end

% set dashed line at y=0
yline(0,'--')

% labels
legend( legEntry(bRange), 'location', 'east')
xlabel('Timestep')
ylabel('Fidelity')

% fix x and y axis
set( gca, 'xtick', 1:6,...
    'xlim', [0 8])
ylim([min(ylim) 1])

% title
titleStr = [type, '\_Certification\_\epsilon=', num2str(epsilon)];
title(titleStr, 'fontweight', 'normal')
FancyPlot(3)

if doSave == 1
    saveStr = [type, '_Certification_epsilon=', num2str(epsilon)];
    saveas( gcf, [FolderS, saveStr, '.pdf'])
end


if ii > nBatches
    figure
    subplot(2,1,1)
    hold on
    plot( 1 : nTimeSteps, p{1}(end,:), 'o-', 'MarkerSize', 6)
    plot( 1 : nTimeSteps, p{2}(end,:), '^--', 'MarkerSize', 6)
    titleStr = [type, '\_Certification\_Parameters'];
    title( titleStr, 'fontweight', 'normal')
    ylabel('Probability')
    legend('p_1','p_2')
    ylim([0 1])
    xlim([.5 nTimeSteps+1.5])
    FancyPlot(3)
    
    subplot(2,1,2)
    hold on
    plot( 1 : nTimeSteps, N{1}(end,:), 'o-', 'MarkerSize', 6)
    plot( 1 : nTimeSteps, N{2}(end,:), '^--', 'MarkerSize', 6)
    xlabel('Timestep')
    ylabel('Counts')
    legend('N_1','N_2')
    ylim([0 max(ylim)])
    xlim([.5 nTimeSteps+1.5])
    FancyPlot(3)
    
    if doSave == 1
        saveStr = [type, '_Certification_Parameters'];
        saveas( gcf, [FolderS, saveStr, '.pdf'])
    end
end
end
end


% plotting functions
function plotWeightfactor( weightfactor)
% plot the weightfactors
figure
clf

m = ['+','o','*','x','s','d','^','v','>','<','p','h'];

[nW, nT] = size( weightfactor);
hold on

for ii = 1 : nW
    plot( 1:nT, weightfactor(ii,:),...
        [m(mod(ii-1,length(m))+1)], 'markersize', 10)
    legEntry{ii} = ['Batch ', num2str(ii)];
end
legend(legEntry, 'Location', 'East');
xlabel('Output channel')
ylabel('Relative weight')
FancyPlot(3)
xlim([0 nT+5])
title('Weightfactor', 'fontweight', 'normal')
end

function plotSingles( sCounts, totSingles, doNormalize, singleNames)
figure()
clf
% first the singles
subplot(3,1,1)
if doNormalize == 1
    bar( sCounts/totSingles)
    ylim([0 1])
    ylabel('Relative singles')
else
    bar( sCounts)
    ylabel('Singles (Hz)')
end
text( 0.05, 0.91, ['Total counts = ', num2str( round(totSingles/1000)), ' kHz'], 'units', 'normalized')
set( gca, 'xtick', 1:length(sCounts), 'xticklabels', singleNames)
box on         
end

function plotCoincidences( cCounts, totCoin, nCoinName, nSingleName, Praw, Pcor, nStates, strState)
subplot(3,1,2)
if doNormalize == 1
    bar( cCounts/totCoin)
    ylabel('Relative coincidences')
else
    bar( cCounts)
    ylabel('Coincidences (Hz)')
end

box on
ylim([0 ceil(max(ylim)*10)/10])
xlim([0 max( nCoinName, nSingleName)+1])
text( 0.05, 0.91, ['Total counts = ', num2str( round(totCoin,2)), ' Hz'], 'units', 'normalized')
xlabel('Output Channel')
set( gca, 'xticklabels', [])


% plot biased probability
subplot(3,1,3)
hold on
bar( -Praw/totCoin)
bar( Pcor/totCoin)
hold off
ylabel('Probability')
xlabel('Output State')
set( gca, 'xtick', 1:nStates, 'xticklabels', strState,...
    'ytick', [-1 0 1], 'yticklabels', {'1', '0', '1'})
legend('Uncorrected', 'Corrected')
ylim([-1 1])
box on
end

function plotPk( Pk_data, nPhotons, nFiles, Modes, type)
% Input
%   Pk_data     Cell array with 1 cell per timestep
%   nPhotons    3
%   nFiles      6 (or 1 for Fourier matrix)
%   nModes      4
%   type        'Canonical'
%               'Fourier'
%               'Identity'
timesteps = [.1 .2 .5 1 2 5];

for ii = Modes
    figure()
    clf
    
    % make a subplot without white space between them
    if nFiles == 1
        nRows = 1;
        nCols = 1;
    else
        nRows = 2;
        nCols = ceil( nFiles/2);
    end
    [ha, ~] = tight_subplot(nRows, nCols, 0, [.18 0.1], [.08 .04]);
    FancyPlot( 3)
    set( gca, 'FontSize', 10)
    
    switch upper(type)
        case 'CANONICAL'
            [Theory,~]= CanonicalState(4,nPhotons);
        case 'FOURIER'
            [Theory,~] = FourierState( 4, nPhotons, ii, 1);
        case 'IDENTITY'
            Theory = [0 1 0 0];
        case 'NONE'
            Theory = [0 0 0 0];
    end
    
    
    for jj = 1 : nFiles
        % select the frame of the subplot
        axes(ha(jj))
        
        
        % plotting
        hold on
        bar(  0 : nPhotons, Theory, 'FaceColor', [200 200 200]/255)
        plot( 0 : nPhotons, Pk_data{jj}(ii,:)', 'o', 'color', 'blue')
        hold off
        
        % setup the axes
        set( gca, 'xtick', 0:nPhotons, 'ytick', [0 .5 1])
        xlim([-.5 nPhotons+.5])
        ylim([0 1.2])
        box on
        
        % setup the time title
        if nFiles > 1
            text( 1.5, 1.13, ['\tau = ', num2str(timesteps(jj),'%3.1f')],...
                'HorizontalAlignment', 'center')
        end
%         [dist,~] = getDistance( Pk_data{jj}(ii,:), Theory);
%         text( -.35, 1.13, ['D = ', num2str( round( dist,3), '%4.3f')])
        
        % set up xticks at the bottom panels
        if jj > nFiles/2 || nFiles == 1
            set( gca, 'xticklabels', {'0', '1', '2', '3'})
            if jj == 5 || nFiles == 1
                xlabel('Photon number k')
            end
        end
        % set up yticks at the left panels
        if mod(jj,nFiles/2) == 1 || nFiles == 1
            set( gca, 'yticklabels', {'0', '0.5', '1.0'})
            ylabel('Probability P(k)')
        end
        FancyPlot(3)
%         % setup the legend
%         if jj == nFiles
%             legend('Canonical', 'Indistinguishable','Distinguishable')
%         end
        
    end
end
end

function plotReferenceHOM(Folder)
% Folder = 'M:\Data\Thermalization\20210521_Certification\Reference HOM dips\';
Folder = 'C:\Users\SomhorstFHB\Downloads\Thermo_data\Data_QuantumPhotoThermodynamics\Fig3_Main\20211119_(In)Distinguishable_Haar+BH\Reference HOM dips\';

figure
clf
subplot(2,1,1)
load([Folder,'20211119_data_Signal1_Idler2.mat']) %20210528_data_Signal1_Idler2.mat
hold on
errorbar( data.PositionList, mean(data.Coincidences.C10C11C9C13),...
    std( data.Coincidences.C10C11C9C13)/sqrt(size(data.Coincidences.C10C11C9C13,1)), 'o')

load([Folder,'20211119_data_Signal1_Signal2.mat'])
errorbar( data.PositionList, mean(data.Coincidences.C10C11C9C13),...
    std( data.Coincidences.C10C11C9C13)/sqrt(size(data.Coincidences.C10C11C9C13,1)), 'o')
hold off
% xlim([5.65 5.85])
ylim([0 1.1*max(ylim)])
ylabel('Counts (Hz)')
legend('S_1-I_2','S_1-S_2')
FancyPlot(3)

subplot(2,1,2)
load([Folder,'20211119_data_Signal2_Idler2.mat'])
errorbar( data.PositionList, mean(data.Coincidences.C10C11/1e3),...
    std( data.Coincidences.C10C11/1e3)/sqrt(size(data.Coincidences.C10C11,1)), 'o')
ylim([0 1.1*max(ylim)])
% xlim([5.45 5.65])
xlabel('Stage position (mm)')
ylabel('Counts (kHz)')
FancyPlot(3)
end


% low level functions
function combs = nmultichoosek(values, k)
    %// Return number of multisubsets or actual multisubsets.
    if numel(values)==1 
        n = values;
        combs = nchoosek(n+k-1,k);
    else
        n = numel(values);
        combs = bsxfun(@minus, nchoosek(1:n+k-1,k), 0:k-1);
        combs = reshape(values(combs),[],k);
    end
end

function [avdDist, maxDist] = getDistance( A, B)
% computes the average distance and max distance

distElement = abs(A-B);
avdDist = mean( distElement);
maxDist = max( distElement);
end

function W = buildTensor( U, HOM1, HOM2, HOM3)
% build the tensor which is used to compute the probability of, given a
% specific input configuration, an output configuration for the netwerk U
% This function follows the definitions as used in Tichy PRL 2015

% HOM1  input mode 2 - 3
% HOM2  input mode 1 - 3
% HOM3  input mode 1 - 2

% setup the distinguishability matrix
S = eye(3);
S = S + sqrt(HOM1) * [0 0 0; 0 0 1; 0 1 0];
S = S + sqrt(HOM2) * [0 0 1; 0 0 0; 1 0 0];
S = S + sqrt(HOM3) * [0 1 0; 1 0 0; 0 0 0];

% get the permutation arrays
Config1 = 1 : length(U);
Config2 = 1 : length(U);

% actually build the nxnxn tensor (n=2)
for kkk = 1 : length(U)
    kValue = Config1(kkk);
    
    for lll = 1 : length(U)
        lValue = Config1(lll);
        
        for jjj = 1 : length(U)
            jValue = Config2(jjj);
            W(kkk,lll,jjj) = U(kValue,jValue) .*conj(U(lValue,jValue))...
                    *S(lValue,kValue);
        end
    end
end
end

function states = getStatesA(microstates)
    % input microstates in format (m1 m2 m3) for a three photon experiemtn 
    % for example. m_i indicates the mode in which every photon is detected

    % returns array of states in format |n1 n2 n3 n4 ... nk>
    % i.e. in the fock basis
    states = {};
    
    for jj = 1:20
        i1=0;i2=0;i3=0;i4=0;
        
        aux = num2str(microstates(jj));
        for ii = 1:3
            ch = aux(ii);
            if ch == '1'
                i1 = i1 + 1;
            end
            if ch == '2'
                i2 = i2 + 1;
            end
            if ch == '3'
                i3 = i3 + 1;
            end
            if ch == '4'
                i4 = i4 + 1;
            end
        end
        
        states{jj} = ['|',num2str(i1),num2str(i2),num2str(i3),num2str(i4),'\rangle'];
        
    end
end
% Functions for figure

function ticks_allowed = getTicksUState();

n_photons = 3;
N_modes = 4;

combs = bsxfun(@minus, nchoosek(1:n_photons+N_modes-1,n_photons), 0:n_photons-1);

res = [];

for ii = 1:size(combs,1)
    label = '';
    for ch = 1:n_photons
        label = append(label, num2str(combs(ii,ch)));
    end
    label = convertCharsToStrings(label);
    
    res(ii) = label;
    
    
end
ticks_allowed = strings(1,20);
states = getStatesA(res(:));
ticks_allowed([1 6 11 17]) = states([1 6 11 17]);

end

function set_ustate_ticks(ax1)

n_photons = 3;
N_modes = 4;

combs = bsxfun(@minus, nchoosek(1:n_photons+N_modes-1,n_photons), 0:n_photons-1);

res = [];

for ii = 1:size(combs,1)
    label = '';
    for ch = 1:n_photons
        label = append(label, num2str(combs(ii,ch)));
    end
    label = convertCharsToStrings(label);
    
    res(ii) = label;
    
    
end
ticks_allowed = cell(1,20);
states = getStatesA(res(:));
for ii = 1:20
    ticks_allowed{ii} = '';
end
for aux  = [1 6 11 17]
    ticks_allowed{aux} = states{aux};
    
end

for tt = 1:6
    axes(ax1(tt))
    set(gca, 'XTick',1:20)
    set(gca, 'XTickLabel', ticks_allowed)
    
    xtickangle(-90)
end

end


function PDist = getPDist(U)
n = 3;
N = 4;

% Setting up input modes
inputmodes = 1:n ;

combs = bsxfun(@minus, nchoosek(1:n+N-1,n), 0:n-1);

length(combs(:,1));
res = [];

for ii = 1:size(combs,1)    
    M = U(inputmodes, combs(ii,:));
    %     combs(ii,:)
    
    %coeffs = BSprobwdist(M);
    
    res(ii,1) = abs(BSProbMicrostate(M))*...
        size(unique(perms(combs(ii,:)),'rows'),1)/factorial(3);
    
end

% PDist = sortrows(res, 'descend');
PDist = res;
PDist(:,1) = PDist(:,1)/sum(PDist(:,1));

end


function P = BSProbMicrostate(M)
    
    M = transpose(M);
    n = size(M,1);
    perm = perms(1:n);
    sum = 0;
    for pp = 1:size(perm, 1)
       sum = sum + permanentRyser(M.*conj(M(perm(pp,:),:)));
    end

    P = sum;
end

% Expectation value for bosons
function P_k = ProbQuantum(M,N)
    
    for k = 0:1:N
        P_k(k+1) = nchoosek(M+N-k-2,N-k)/nchoosek(M+N-1,N);
    end
    
end

function plot_FidelitiesLB_BH(data)
global scl
% Making vacuum, 1,2 & 3 photon state vectors
vac = [1;0;0;0];
ph1 = [0;1;0;0];
ph2 = [0;0;1;0];
ph3 = [0;0;0;1];

in = kron(ph1,kron(ph1,kron(ph1,vac))); % |1110>

nt = 1000; 
tvec = linspace(0,2*pi,nt);

for j = 1:nt

out = BH4(tvec(j),4,1)*in;

[s,u,v] = SchmidtDecomposition(out,[16,16],0); % Even bipartition (subsystem A=modes 1 & 2, subsystem B = modes 3& 4

smax(j) = s(1);
S2(j) = -sum(abs(s).^2.*log2(abs(s).^2)); % Entanglement entropy

[s,u,v] = SchmidtDecomposition(out,[4,64],0); % A = mode 1, B = modes 1,2,&3 

smax1(j) = s(1);
S1(j) = -sum(abs(s).^2.*log2(abs(s).^2)); %Entanglement entropy
end


color_line = [.9 0 0 .9];

hold on
%semilogx(tvec,smax1.^2, '--','Color',color_line,'LineWidth',2)
semilogx(tvec,smax.^2,':','Color',color_line,'LineWidth',2.5)


alpha(.7)

color_points = [0.9290, 0.6940, 0.1250];


F_BH = [0.2689    0.2938    0.2105    0.3587    0.1804    0.2034];
dF_BH = [0.0089    0.0100    0.0167    0.0048     0.0099    0.0101];


data_y = data(2,:);
err_yneg = data(2,:) - data(3,:);
err_ypos = data(2,:) - data(1,:);

errorbar([.1 .2 .5 1 2 5], data_y, err_yneg, err_ypos, ...% dF_BH,...
    '+', 'LineWidth', 1.7, ...
    'MarkerSize',10,'Capsize', 10,...
    'Color', 'k',...
    'MarkerEdgeColor', 'k','MarkerFaceColor', color_points	)

hold on

 
xMin = tvec+1E-6;
xMax = flip( tvec)+1E-6;
yMin = zeros(1,length(tvec));
yMax = 1.1*ones(1, length(tvec));
cRange = [smax.^2 flip(smax.^2)];
fill([xMin xMax], [yMin, yMax], 1-[cRange],...
    'FaceAlpha', 0.18, 'edgecolor', 'none', 'HandleVisibility', 'off');

 
nSteps = 50;    % higher = smoother
colorMap = [ones(nSteps,1), linspace(1,0,nSteps)', linspace(1,0,nSteps)'];
colormap(colorMap);


hold off



end


 
function plot_FidelitiesLB_Haar(data)
global scl
nt = 100;
tvec = linspace(0,2*pi,nt);%[.1,.2,.5,1,2,5];

U_23 = [0.916301887235175-0.234423078839647i,0.222818807633934+0.083264838525007i,-0.189508860852433+0.00459546037923568i,0.11339130138798-0.00794376223520768i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i
-0.190574596932734+0.130391639645809i,0.931012251969148-0.0252191905677434i,-0.0949976961791949-0.140030013692065i,-0.165176569041326-0.152785000738382i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i
0.18624945715815-0.0171842040255126i,0.11286928699168-0.124195818416443i,0.963086535079646-0.0764683550003624i,-0.0587658981177007-0.00390530823428146i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i
-0.120796890432933+0.0503981048556171i,0.163797044233778-0.144932314487063i,0.0443917307121204-0.00886550983107418i,0.95120763655892+0.167893132492689i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i
0+0i,0+0i,0+0i,0+0i,1+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i
0+0i,0+0i,0+0i,0+0i,0+0i,1+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i
0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,1+0i,0+0i,0+0i,0+0i,0+0i,0+0i
0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,1+0i,0+0i,0+0i,0+0i,0+0i
0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,1+0i,0+0i,0+0i,0+0i
0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,1+0i,0+0i,0+0i
0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,1+0i,0+0i
0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,0+0i,1+0i];

for j = 1:nt
%     out = Ubuild(mpower(U_23,tvec(j)),4);
    out = buildFockOut(mpower(U_23(1:4,1:4),tvec(j)),[1 1 1 0],4);
    
    [s,u,v] = SchmidtDecomposition(out,[16,16],0);

    smax(j) = s(1);
    S2(j) = -sum(abs(s).^2.*log2(abs(s).^2));

    [s,u,v] = SchmidtDecomposition(out,[4,64],0);

    smax1(j) = s(1);
    S1(j) = -sum(abs(s).^2.*log2(abs(s).^2));
end

color_line = [.9 0 0 .9];

hold on
% semilogx(tvec,smax1.^2, '--','Color',color_line,'LineWidth',2) %we never cross this limit so why plot it .  
semilogx(tvec,smax.^2,':','Color',color_line,'LineWidth',2.5)

alpha(.7)

color_points = [0.9290, 0.6940, 0.1250 ];

F_H = [0.3208    0.4631    0.3197    0.2324    0.3631    0.0622];
dF_H = [0.0065    0.0119    0.0139    0.0116    0.0143    0.0168];

data_y = data(2,:);
err_yneg = data(2,:) - data(3,:);
err_ypos = data(2,:) - data(1,:);

errorbar([.1 .2 .5 1 2 5], data_y, err_yneg, err_ypos, ...% dF_H,...
    '+', 'LineWidth', 1.7, ...
    'MarkerSize',10,'Capsize', 10,...
    'Color', 'k',...
    'MarkerEdgeColor', 'k','MarkerFaceColor', color_points	)

hold on


xMin = tvec+1E-6;
xMax = flip( tvec)+1E-6;
yMin = zeros(1,length(tvec));
yMax = 1.1*ones(1, length(tvec));
cRange = [smax.^2 flip(smax.^2)];
fill([xMin xMax], [yMin, yMax], 1-[cRange],...
    'FaceAlpha', 0.17, 'edgecolor', 'none', 'HandleVisibility', 'off');


nSteps = 50;    % higher = smoother
colorMap = [ones(nSteps,1), linspace(1,0,nSteps)', linspace(1,0,nSteps)'];
colormap(colorMap);

% semilogx([.1 .2 .5 1 2 5], [.766 .734 .684 .582 .652 .675], 'x', 'MarkerSize', 10, 'LineWidth', 3)





hold off

end





function plot_hand = plot_cloud_plots(Pk_data, nPhotons, nFiles, HOM, TheoryType, type, ax)

global scl
global ytickangl
marker_factor = 1.8;

if strcmp(type,'Haar')
    marker_factor = 1.15;
end

color = [0.9020    0.9020    0.9020];

switch upper(TheoryType)
    case 'CANONICAL'
        [Theory]= CanonicalState(4,nPhotons);
    case 'FOURIER'
        [Theory,~] = FourierState( 4, nPhotons, ii, 1);
    case 'IDENTITY'
        Theory = [0 1 0 0];
    otherwise
        Theory = [0 0 0 0];
end


face_green = [0 .8 0];
edge_green = [0 .5 0];

for jj = 1 : nFiles
    
    mode_selected_show = 1; %hardcode selected mode 1 wlg
    
    
    ii = mode_selected_show;
    
    % select the frame of the subplot
    axes(ax(jj));
    
    
    % plot theory background
    hold on
    br = bar(  0 : nPhotons, Theory, 'FaceColor', color);
    br.BaseValue = 0.02;
    
    % loop through all types of data
    sNames = fieldnames(Pk_data);
    nNames = length(sNames);
    
    for ss = 1 : nNames
        
        % in case of all Haar plotting
        if size( Pk_data.(sNames{ss}),2) == 120
            maxSteps = 20;
        else
            maxSteps = 1;
        end
        
        % plot the individual markers just next to eachother
        xOffset = linspace(.05,.25,maxSteps);
        
        % dist. left,  indst. right & setup the markers
        lineSpecl = {'o', 'Color', '#f25e5e', 'MarkerFaceColor',...
            '#f25e5e', 'MarkerSize', marker_factor*4*scl, 'HandleVisibility', 'on'};
        lineSpec2 = {'o', 'Color', '#42a1f5', 'MarkerFaceColor',...
            '#42a1f5', 'MarkerSize', marker_factor*4*scl,'HandleVisibility', 'on'};
        lineSpec3 =  {'o', 'Color', 'black', 'MarkerFaceColor',...
            'black', 'MarkerSize', marker_factor*2*scl, 'HandleVisibility', 'on'};
        lineSpec4 = {'o', 'Color', 'black', 'MarkerFaceColor',...
            'black', 'MarkerSize', marker_factor*2*scl, 'HandleVisibility', 'on'};
        
        plot_hand = [];
        green_spec = [0.3922    0.8314    0.0745];
        % now start plotting
        k_range = maxSteps:-1:1;
        if ss == 1
            for kk = (jj-1)*maxSteps + (k_range)
                % plot exp data
                xOffInd = mod(kk-1,maxSteps)+1;
                if mod(kk,20) == 1
                    xOffset(xOffInd) = 0.2;
                end
                plot_hand(1) = plot( (0 : nPhotons)+xOffset(xOffInd),...
                    Pk_data.(sNames{ss}){kk}(ii,:)', lineSpecl{:});
                
            end
            
        else
            for kk = (jj-1)*maxSteps + (k_range)
                % plot exp data, distinguishable
                xOffInd = mod(kk-1,maxSteps)+1;
                if mod(kk,20) == 1
                    xOffset(xOffInd) = 0.2;
                end
                U = loadIdealUnitary( type, kk);
                Pk_dist = getNumericalSimulation(U, ii, 4, 3, 0, 0, 0, 0);
                
                plot_hand(2) = plot( (0 : nPhotons)-xOffset(xOffInd),...
                    Pk_data.(sNames{ss}){kk}(ii,:)', lineSpec2{:});
                
                
            end
            
            for kk = (jj-1)*maxSteps + (k_range)
                % plot theory, indistinguishable
                xOffInd = mod(kk-1,maxSteps)+1;
                U = loadIdealUnitary( type, kk);
                Pk_indist = getNumericalSimulation(U, ii, 4, 3,...
                    sqrt(HOM(3)), sqrt(HOM(1)), sqrt(HOM(2)), 0);
                
                plot_hand(3) = plot( (0 : nPhotons)+xOffset(xOffInd),...
                    Pk_indist, lineSpec3{:});
            end
            
            for kk = (jj-1)*maxSteps + (k_range)
                % plot num data, distinguishable
                xOffInd = mod(kk-1,maxSteps)+1;
                U = loadIdealUnitary( type, kk);
                Pk_dist = getNumericalSimulation(U, ii, 4, 3, 0, 0, 0, 0);
                
                plot_hand(4) = plot( (0 : nPhotons)-xOffset(xOffInd),...
                    Pk_dist, lineSpec4{:});
            end
        end
        
        
        
        
    end
    
    hold off
    
    % Set the ticks correctly on both sides
    
    tck = gca;
    
    tck.XTick = [0:3];
    tck.FontSize = 18*scl;
    tck.XRuler.TickLabelGapOffset = -7;
    
    xlim([-.55 3.55])
    ylim([0 1.1])
    if jj == 99
        xlab = xlabel('Photon Number');
        xlab.Position(1) = -3*scl;
    end
    
    if jj == 1
        ylabel('Probability P(k)','interpreter','TeX', 'Fontsize', 30)
    end
    
    ytickangle(ytickangl)
    %
    % if jj ~= 1
    %     set(gca,'yticklabel',[])
    % end
    % if jj == 126
    %     yyaxis right
    %     set( gca, 'YColor', 'k' )
    %     set(gca,'yticklabel',[])
    %
    %     yyaxis left
    %     set(gca,'yticklabel',[])
    %     set( gca, 'YColor', 'k' )
    % end
    
    
    % setup the legend
    if jj == nFiles
        hold on
        plot( NaN, NaN, lineSpecl{:}, 'HandleVisibility', 'on')
        plot( NaN, NaN, lineSpec2{:}, 'HandleVisibility', 'on')
        
        hold off
    end
end
end

function U = loadIdealUnitary( type, number, Folder)
% imports the unitary matrix
%
% Input
%   type        The matrix type ('Haar' or 'BoseHubbard')
%   number      The matrix number
%
% optional:
%   Folder      the path where you can find the data
%
% Output:
%   U           A nxn unitary matrix

% set default folder or verify Folder, whichever applies
if nargin == 2
    Folder = [pwd,'\20211119_(In)Distinguishable_Haar+BH\Matrices\'];
elseif ~strcmp( Folder(end), '\')
    Folder = [Folder, '\'];
end

% select the filename
if strcmp( upper(type), 'BOSEHUBBARD')
    filename = ['Matrix_Hubbard_',num2str(number),'_U.txt.txt'];
    
elseif strcmp( upper(type), 'HAAR')
    filename = ['Matrix_Haar_',num2str(number),'_U.txt.txt'];
end

U2 = loadComplexNumbersTxt( Folder, filename);
U = U2(1:4,1:4);

end


function Pk = getNumericalSimulation(U, mode_of_interest, nModes, nPhotons, x_12,x_23,x_31,phi)
% Numerical boson sampling with partial distinguishability for N = 3
% photons in M = 4 modes. The function returns Pk distribution for the
% ideal matrices with imperfect photons, for reference, see Tichy's:
%   "Distinguishability and many-particle interference"
%   (https://arxiv.org/pdf/1609.09804.pdf)
%
% Input parameters
%   U                   Unitary of interest, col = input, row = output
%   mode_of_interest    The specific mode of which you want the marginal
%                       probability distribution Pk
%   nModes              # modes of the system
%   nPhotons            # photons of the system
%   x_12                sqrt(HOM) between photons of mode 1 and 2
%   x_23                sqrt(HOM) between photons of mode 2 and 3
%   x_31                sqrt(HOM) between photons of mode 3 and 1
%   phi                 free parameters for triad phase

%
% function by Frank Somhorst

%M = transpose(U); % Adapt to Tichy et al. convention
M = U;

% Create mode assignment list of all possible microstates for N = 3 photons
% and M = 4 modes system.
microstate_list = [];
for ii = 1:4
    for jj = ii:4
        for kk = jj:4
            microstate_list(end+1,:) = [ii jj kk];
        end
    end
end

% Calculate all possible input photon permutations
permIn = perms(1:3);

A = zeros(20,6); % Store for 20 microstates

for ii = 1 : size(microstate_list,1)
    A(ii,1) = perm(M(1:3,microstate_list(ii,:)).*conj(M(permIn(6,:),microstate_list(ii,:)))) ...
        * size(unique(perms(microstate_list(ii,:)),'rows'),1)/factorial(3);
    
    A(ii,2) = perm(M(1:3,microstate_list(ii,:)).*conj(M(permIn(4,:),microstate_list(ii,:)))) ...
        * size(unique(perms(microstate_list(ii,:)),'rows'),1)/factorial(3);
    
    A(ii,3) = perm(M(1:3,microstate_list(ii,:)).*conj(M(permIn(5,:),microstate_list(ii,:)))) ...
        * size(unique(perms(microstate_list(ii,:)),'rows'),1)/factorial(3);
    
    A(ii,4) = perm(M(1:3,microstate_list(ii,:)).*conj(M(permIn(1,:),microstate_list(ii,:)))) ...
        * size(unique(perms(microstate_list(ii,:)),'rows'),1)/factorial(3);
    
    A(ii,5) = perm(M(1:3,microstate_list(ii,:)).*conj(M(permIn(3,:),microstate_list(ii,:)))) ...
        * size(unique(perms(microstate_list(ii,:)),'rows'),1)/factorial(3);
    
    A(ii,6) = perm(M(1:3,microstate_list(ii,:)).*conj(M(permIn(2,:),microstate_list(ii,:)))) ...
        * size(unique(perms(microstate_list(ii,:)),'rows'),1)/factorial(3);
    
end

x_vector = [1 x_12^2 x_23^2 x_31^2 x_12*x_23*x_31*exp(1i*phi) x_12*x_23*x_31*exp(-1i*phi)]';

P_microstate =  real(A*x_vector); % because saved as complex number, zero imaginary part still gives warnings!

Pk = zeros(1,4);
states = getStates(3,4);

for k = 0:1:3
    for ii = 1:length(states)
        if states(ii,mode_of_interest) == k
            Pk(k+1) = Pk(k+1) + P_microstate(ii,1);
        end
    end
end

end



function plotMicroStateCertification( Pcor, batchRange, nBatches,ax,...
    nStates, nTimeSteps, type,MatType, sName,  doSave, FolderS)
global scl

if strcmp(type,'U1')
    nn = 2;
elseif strcmp(type,'U1F')
    nn = 3;
elseif strcmp(type,'U')
    nn = 1;
end

timesteps = [.1 .2 .5 1 2 5];

offset = 0.35;
offsets = linspace(-offset,offset,6);

% loop through all certification measurements\

% loop through all timesteps
for ii = 1 : nTimeSteps
    
    axes(ax(ii));
    % loop through all batches
    for bb = batchRange
        %
        %          if length(sName) ~= 2
        % %              get the relevant Pcor data
        %              if bb <= nBatches
        %                  yData = Pcor.(sName{nn}){ii}(bb,:);
        %              else
        %                  yData = sum(Pcor.(sName{nn}){ii});
        %              end
        
        if strcmp(MatType,'BoseHubbard')
            % get the relevant Pcor data
            if bb <= nBatches
                disp(bb)
                yData = Pcor.(sName{nn}){ii}(bb,:);
            else
                yData = sum(Pcor.(sName{nn}){ii});
            end
            
        elseif (strcmp(MatType, 'Haar')) & (strcmp(type, 'U')) == 1
            % get the relevant Pcor data
            if bb <= nBatches
                yData = Pcor.(sName{nn}){(ii-1)*20+1}(bb,:);
            else
                yData = sum(Pcor.(sName{nn}){ii});
            end
        elseif (strcmp(MatType, 'Haar')) & (strcmp(type, 'U')) ~= 1
            % get the relevant Pcor data
            if bb <= nBatches
                yData = Pcor.(sName{nn}){ii}(bb,:);
            else
                yData = sum(Pcor.(sName{nn}){ii});
            end
        else
            warning('No match found for Matrix and Measurement type')
        end
        
        hold on
        color = [0 0 0];
        
      
        color_g_soft   = [0 .9 0 ];
        
        color_g_strong = [0 .5 0];
        color_r_soft   = [.8 0 0 ];
        color_r_strong = [.5 0 0];
        
        if strcmp(type, 'U')& bb == batchRange(end)
            
            pl = plot( [1:nStates], yData / sum(yData), 'o',...
                'Color', color, 'MarkerSize', 4.5*scl, 'LineWidth', .7*scl);
            
            %Dummy data for the legend
              pl = scatter( [], [], 35*scl, 'o',...
                'MarkerEdgeColor', color_r_strong, 'LineWidth', 1*scl,...
                'MarkerFaceColor', color_r_soft, 'MarkerFaceAlpha', .1);
            scatter( [], [] / sum(yData), 35*scl, 'o',...
                'MarkerEdgeColor', color_g_strong,  'LineWidth', 1*scl,...
                'MarkerFaceColor', color_g_soft, 'MarkerFaceAlpha', .1);
            
            hold off
            
        elseif strcmp(type, 'U1') & bb == batchRange(end)
            hold on
            
            pl = scatter( [1:nStates], yData / sum(yData), 35*scl, 'o',...
                'MarkerEdgeColor', color_r_strong, 'LineWidth', 1*scl,...
                'MarkerFaceColor', color_r_soft, 'MarkerFaceAlpha', .1);
            scatter( [6], yData(6) / sum(yData), 35*scl, 'o',...
                'MarkerEdgeColor', color_g_strong,  'LineWidth', 1*scl,...
                'MarkerFaceColor', color_g_soft, 'MarkerFaceAlpha', .1);
            
            hold off
            
        elseif strcmp(type, 'U1F') & bb == batchRange(end)
            hold on
            
            pl = scatter( [1:nStates], yData / sum(yData), 35*scl, 'o',...
                'MarkerEdgeColor', color_r_strong, 'LineWidth', 1*scl,...
                'MarkerFaceColor', color_r_soft, 'MarkerFaceAlpha', .1);
            scatter( [1 6 11 17], yData([1 6 11 17]) / sum(yData), 35*scl, 'o',...
                'MarkerEdgeColor', color_g_strong,  'LineWidth', 1*scl,...
                'MarkerFaceColor', color_g_soft, 'MarkerFaceAlpha', .1);
            
            set(gca, 'XTick', [1 5 10 15])
            
            hold off
            
            
        else
            warning("No data plotted")
        end
        

        
        
   
    
end
 
end






end



function dist = GetL1Dist(Pk_data, nPhotons, TheoryType, type)

% Compute the theory probability distribution
switch upper(TheoryType)
    case 'CANONICAL'
        [Theory]= CanonicalState(4,nPhotons);
    case 'FOURIER'
        [Theory,~] = FourierState( 4, nPhotons, ii, 1);
    case 'IDENTITY'
        Theory = [0 1 0 0];
    otherwise
        Theory = [0 0 0 0];
end

nMat = 1;
if strcmp(upper(type), 'HAAR');
    nMat = 20;
end
nTimesteps = 6;


% loop through all types of data
sNames = fieldnames(Pk_data);
nNames = length(sNames);


% Array that will contain the L1 distances for every matrix
dist = zeros(nNames ,nTimesteps, nMat);

for ss = 1 : nNames
    
    mode_selected_show = 1; %hardcode selected mode 1 wlg   
    ii = mode_selected_show;
    
    for tt = 1:nTimesteps
        
        for nn = 1:nMat
            kk = (tt-1)*nMat + nn;
            P_exp = Pk_data.(sNames{ss}){kk}(ii,:)';
            D = TotalVariationDist(Theory, ...
                                   P_exp);
                               
            dist(ss, tt, nn) = D;
        end
       
    end
end
end
   

function TV = TotalVariationDist(P1, P2)
% Computes the total variation distance (L1 norm) between two probability
% distributions

if length(P1) ~= length(P2)
    error('Arrays must be same length')
end

TV = 0;

for ii = 1:length(P1)
    TV = TV + abs(P1(ii) - P2(ii));
end

TV = TV*0.5;

end



